unit class Mat44 is Array;

method perl () { 'Mat44.new(' ~ join(',', self.list».perl) ~ ')' }

method dump () { say self.perl }

sub mat44 (|a) is export { Mat44.new(|a) }

multi sub mat44-zero () is export { Mat44.new(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0) };

multi sub mat44-ident () is export { Mat44.new(1,1,1,1,1,1,1,1) };

method AT-POS ($i) is rw {
    self.Array::AT-POS($_  ),
    self.Array::AT-POS($_+1),
    self.Array::AT-POS($_+2),
    self.Array::AT-POS($_+3)
        given $i*4;
}

multi sub mat44-trans (Numeric:D $a, Numeric:D $b, Numeric:D $c --> Mat44:D) is pure is export {
    Mat44.new(
        1,
        0,
        0,
        $a,
        0,
        1,
        0,
        $b,
        0,
        0,
        1,
        $c,
        0,
        0,
        0,
        1
    );
}

multi method mat44-trans (Numeric:D $a, Numeric:D $b, Numeric:D $c --> Mat44:D) is pure {
    Mat44.new(
        1,
        0,
        0,
        $a,
        0,
        1,
        0,
        $b,
        0,
        0,
        1,
        $c,
        0,
        0,
        0,
        1
    );
}

multi sub mat44-trans (Numeric:D $a, Numeric:D $b, Numeric:D $c, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        1,
        0,
        0,
        $a,
        0,
        1,
        0,
        $b,
        0,
        0,
        1,
        $c,
        0,
        0,
        0,
        1;
    $r;
}

multi method mat44-trans (Numeric:D $a, Numeric:D $b, Numeric:D $c, Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        1,
        0,
        0,
        $a,
        0,
        1,
        0,
        $b,
        0,
        0,
        1,
        $c,
        0,
        0,
        0,
        1;
    $r;
}

multi sub mat44-scale (Numeric:D $a, Numeric:D $b, Numeric:D $c --> Mat44:D) is pure is export {
    Mat44.new(
        $a,
        0,
        0,
        0,
        0,
        $b,
        0,
        0,
        0,
        0,
        $c,
        0,
        0,
        0,
        0,
        1
    );
}

multi method mat44-scale (Numeric:D $a, Numeric:D $b, Numeric:D $c --> Mat44:D) is pure {
    Mat44.new(
        $a,
        0,
        0,
        0,
        0,
        $b,
        0,
        0,
        0,
        0,
        $c,
        0,
        0,
        0,
        0,
        1
    );
}

multi sub mat44-scale (Numeric:D $a, Numeric:D $b, Numeric:D $c, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a,
        0,
        0,
        0,
        0,
        $b,
        0,
        0,
        0,
        0,
        $c,
        0,
        0,
        0,
        0,
        1;
    $r;
}

multi method mat44-scale (Numeric:D $a, Numeric:D $b, Numeric:D $c, Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a,
        0,
        0,
        0,
        0,
        $b,
        0,
        0,
        0,
        0,
        $c,
        0,
        0,
        0,
        0,
        1;
    $r;
}

multi sub mat44-rot-x (Numeric:D $a --> Mat44:D) is pure is export {
    my ($sin, $cos) = sin($a), cos($a);

    Mat44.new(
        1,
        0,
        0,
        0,
        0,
        $cos,
        -$sin,
        0,
        0,
        $sin,
        $cos,
        0,
        0,
        0,
        0,
        1
    );
}

multi method mat44-rot-x (Numeric:D $a --> Mat44:D) is pure {
    my ($sin, $cos) = sin($a), cos($a);

    Mat44.new(
        1,
        0,
        0,
        0,
        0,
        $cos,
        -$sin,
        0,
        0,
        $sin,
        $cos,
        0,
        0,
        0,
        0,
        1
    );
}

multi sub mat44-rot-x (Numeric:D $a, Mat44:D $r is rw --> Mat44:D) is export {
    my ($sin, $cos) = sin($a), cos($a);

    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        1,
        0,
        0,
        0,
        0,
        $cos,
        -$sin,
        0,
        0,
        $sin,
        $cos,
        0,
        0,
        0,
        0,
        1;
    $r;
}

multi method mat44-rot-x (Numeric:D $a, Mat44:D $r is rw --> Mat44:D) {
    my ($sin, $cos) = sin($a), cos($a);

    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        1,
        0,
        0,
        0,
        0,
        $cos,
        -$sin,
        0,
        0,
        $sin,
        $cos,
        0,
        0,
        0,
        0,
        1;
    $r;
}

multi sub mat44-rot-y (Numeric:D $a --> Mat44:D) is pure is export {
    my ($sin, $cos) = sin($a), cos($a);

    Mat44.new(
        $cos,
        0,
        -$sin,
        0,
        0,
        1,
        0,
        0,
        $sin,
        0,
        $cos,
        0,
        0,
        0,
        0,
        1
    );
}

multi method mat44-rot-y (Numeric:D $a --> Mat44:D) is pure {
    my ($sin, $cos) = sin($a), cos($a);

    Mat44.new(
        $cos,
        0,
        -$sin,
        0,
        0,
        1,
        0,
        0,
        $sin,
        0,
        $cos,
        0,
        0,
        0,
        0,
        1
    );
}

multi sub mat44-rot-y (Numeric:D $a, Mat44:D $r is rw --> Mat44:D) is export {
    my ($sin, $cos) = sin($a), cos($a);

    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $cos,
        0,
        -$sin,
        0,
        0,
        1,
        0,
        0,
        $sin,
        0,
        $cos,
        0,
        0,
        0,
        0,
        1;
    $r;
}

multi method mat44-rot-y (Numeric:D $a, Mat44:D $r is rw --> Mat44:D) {
    my ($sin, $cos) = sin($a), cos($a);

    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $cos,
        0,
        -$sin,
        0,
        0,
        1,
        0,
        0,
        $sin,
        0,
        $cos,
        0,
        0,
        0,
        0,
        1;
    $r;
}

multi sub mat44-rot-z (Numeric:D $a --> Mat44:D) is pure is export {
    my ($sin, $cos) = sin($a), cos($a);

    Mat44.new(
        $cos,
        -$sin,
        0,
        0,
        $sin,
        $cos,
        0,
        0,
        0,
        0,
        1,
        0,
        0,
        0,
        0,
        1
    );
}

multi method mat44-rot-z (Numeric:D $a --> Mat44:D) is pure {
    my ($sin, $cos) = sin($a), cos($a);

    Mat44.new(
        $cos,
        -$sin,
        0,
        0,
        $sin,
        $cos,
        0,
        0,
        0,
        0,
        1,
        0,
        0,
        0,
        0,
        1
    );
}

multi sub mat44-rot-z (Numeric:D $a, Mat44:D $r is rw --> Mat44:D) is export {
    my ($sin, $cos) = sin($a), cos($a);

    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $cos,
        -$sin,
        0,
        0,
        $sin,
        $cos,
        0,
        0,
        0,
        0,
        1,
        0,
        0,
        0,
        0,
        1;
    $r;
}

multi method mat44-rot-z (Numeric:D $a, Mat44:D $r is rw --> Mat44:D) {
    my ($sin, $cos) = sin($a), cos($a);

    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $cos,
        -$sin,
        0,
        0,
        $sin,
        $cos,
        0,
        0,
        0,
        0,
        1,
        0,
        0,
        0,
        0,
        1;
    $r;
}

multi sub add (Mat44:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] + $b[0][0],
        $a[0][1] + $b[0][1],
        $a[0][2] + $b[0][2],
        $a[0][3] + $b[0][3],
        $a[1][0] + $b[1][0],
        $a[1][1] + $b[1][1],
        $a[1][2] + $b[1][2],
        $a[1][3] + $b[1][3],
        $a[2][0] + $b[2][0],
        $a[2][1] + $b[2][1],
        $a[2][2] + $b[2][2],
        $a[2][3] + $b[2][3],
        $a[3][0] + $b[3][0],
        $a[3][1] + $b[3][1],
        $a[3][2] + $b[3][2],
        $a[3][3] + $b[3][3]
    );
}

multi method add (Mat44:D $a: Mat44:D $b --> Mat44:D) is pure {
    Mat44.new(
        $a[0][0] + $b[0][0],
        $a[0][1] + $b[0][1],
        $a[0][2] + $b[0][2],
        $a[0][3] + $b[0][3],
        $a[1][0] + $b[1][0],
        $a[1][1] + $b[1][1],
        $a[1][2] + $b[1][2],
        $a[1][3] + $b[1][3],
        $a[2][0] + $b[2][0],
        $a[2][1] + $b[2][1],
        $a[2][2] + $b[2][2],
        $a[2][3] + $b[2][3],
        $a[3][0] + $b[3][0],
        $a[3][1] + $b[3][1],
        $a[3][2] + $b[3][2],
        $a[3][3] + $b[3][3]
    );
}

multi sub add (Mat44:D $a, Mat44:D $b, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] + $b[0][0],
        $a[0][1] + $b[0][1],
        $a[0][2] + $b[0][2],
        $a[0][3] + $b[0][3],
        $a[1][0] + $b[1][0],
        $a[1][1] + $b[1][1],
        $a[1][2] + $b[1][2],
        $a[1][3] + $b[1][3],
        $a[2][0] + $b[2][0],
        $a[2][1] + $b[2][1],
        $a[2][2] + $b[2][2],
        $a[2][3] + $b[2][3],
        $a[3][0] + $b[3][0],
        $a[3][1] + $b[3][1],
        $a[3][2] + $b[3][2],
        $a[3][3] + $b[3][3];
    $r;
}

multi method add (Mat44:D $a: Mat44:D $b, Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] + $b[0][0],
        $a[0][1] + $b[0][1],
        $a[0][2] + $b[0][2],
        $a[0][3] + $b[0][3],
        $a[1][0] + $b[1][0],
        $a[1][1] + $b[1][1],
        $a[1][2] + $b[1][2],
        $a[1][3] + $b[1][3],
        $a[2][0] + $b[2][0],
        $a[2][1] + $b[2][1],
        $a[2][2] + $b[2][2],
        $a[2][3] + $b[2][3],
        $a[3][0] + $b[3][0],
        $a[3][1] + $b[3][1],
        $a[3][2] + $b[3][2],
        $a[3][3] + $b[3][3];
    $r;
}

multi sub plus (Mat44:D $a, Mat44:D $b --> Mat44:D) is export {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] + $b[0][0],
        $a[0][1] + $b[0][1],
        $a[0][2] + $b[0][2],
        $a[0][3] + $b[0][3],
        $a[1][0] + $b[1][0],
        $a[1][1] + $b[1][1],
        $a[1][2] + $b[1][2],
        $a[1][3] + $b[1][3],
        $a[2][0] + $b[2][0],
        $a[2][1] + $b[2][1],
        $a[2][2] + $b[2][2],
        $a[2][3] + $b[2][3],
        $a[3][0] + $b[3][0],
        $a[3][1] + $b[3][1],
        $a[3][2] + $b[3][2],
        $a[3][3] + $b[3][3];
    $a;
}

multi method plus (Mat44:D $a: Mat44:D $b --> Mat44:D) {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] + $b[0][0],
        $a[0][1] + $b[0][1],
        $a[0][2] + $b[0][2],
        $a[0][3] + $b[0][3],
        $a[1][0] + $b[1][0],
        $a[1][1] + $b[1][1],
        $a[1][2] + $b[1][2],
        $a[1][3] + $b[1][3],
        $a[2][0] + $b[2][0],
        $a[2][1] + $b[2][1],
        $a[2][2] + $b[2][2],
        $a[2][3] + $b[2][3],
        $a[3][0] + $b[3][0],
        $a[3][1] + $b[3][1],
        $a[3][2] + $b[3][2],
        $a[3][3] + $b[3][3];
    $a;
}

multi sub infix:<+> (Mat44:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] + $b[0][0],
        $a[0][1] + $b[0][1],
        $a[0][2] + $b[0][2],
        $a[0][3] + $b[0][3],
        $a[1][0] + $b[1][0],
        $a[1][1] + $b[1][1],
        $a[1][2] + $b[1][2],
        $a[1][3] + $b[1][3],
        $a[2][0] + $b[2][0],
        $a[2][1] + $b[2][1],
        $a[2][2] + $b[2][2],
        $a[2][3] + $b[2][3],
        $a[3][0] + $b[3][0],
        $a[3][1] + $b[3][1],
        $a[3][2] + $b[3][2],
        $a[3][3] + $b[3][3]
    );
}

multi sub add (Mat44:D $a, Numeric:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] + $b,
        $a[0][1] + $b,
        $a[0][2] + $b,
        $a[0][3] + $b,
        $a[1][0] + $b,
        $a[1][1] + $b,
        $a[1][2] + $b,
        $a[1][3] + $b,
        $a[2][0] + $b,
        $a[2][1] + $b,
        $a[2][2] + $b,
        $a[2][3] + $b,
        $a[3][0] + $b,
        $a[3][1] + $b,
        $a[3][2] + $b,
        $a[3][3] + $b
    );
}

multi method add (Mat44:D $a: Numeric:D $b --> Mat44:D) is pure {
    Mat44.new(
        $a[0][0] + $b,
        $a[0][1] + $b,
        $a[0][2] + $b,
        $a[0][3] + $b,
        $a[1][0] + $b,
        $a[1][1] + $b,
        $a[1][2] + $b,
        $a[1][3] + $b,
        $a[2][0] + $b,
        $a[2][1] + $b,
        $a[2][2] + $b,
        $a[2][3] + $b,
        $a[3][0] + $b,
        $a[3][1] + $b,
        $a[3][2] + $b,
        $a[3][3] + $b
    );
}

multi sub add (Mat44:D $a, Numeric:D $b, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] + $b,
        $a[0][1] + $b,
        $a[0][2] + $b,
        $a[0][3] + $b,
        $a[1][0] + $b,
        $a[1][1] + $b,
        $a[1][2] + $b,
        $a[1][3] + $b,
        $a[2][0] + $b,
        $a[2][1] + $b,
        $a[2][2] + $b,
        $a[2][3] + $b,
        $a[3][0] + $b,
        $a[3][1] + $b,
        $a[3][2] + $b,
        $a[3][3] + $b;
    $r;
}

multi method add (Mat44:D $a: Numeric:D $b, Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] + $b,
        $a[0][1] + $b,
        $a[0][2] + $b,
        $a[0][3] + $b,
        $a[1][0] + $b,
        $a[1][1] + $b,
        $a[1][2] + $b,
        $a[1][3] + $b,
        $a[2][0] + $b,
        $a[2][1] + $b,
        $a[2][2] + $b,
        $a[2][3] + $b,
        $a[3][0] + $b,
        $a[3][1] + $b,
        $a[3][2] + $b,
        $a[3][3] + $b;
    $r;
}

multi sub plus (Mat44:D $a, Numeric:D $b --> Mat44:D) is export {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] + $b,
        $a[0][1] + $b,
        $a[0][2] + $b,
        $a[0][3] + $b,
        $a[1][0] + $b,
        $a[1][1] + $b,
        $a[1][2] + $b,
        $a[1][3] + $b,
        $a[2][0] + $b,
        $a[2][1] + $b,
        $a[2][2] + $b,
        $a[2][3] + $b,
        $a[3][0] + $b,
        $a[3][1] + $b,
        $a[3][2] + $b,
        $a[3][3] + $b;
    $a;
}

multi method plus (Mat44:D $a: Numeric:D $b --> Mat44:D) {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] + $b,
        $a[0][1] + $b,
        $a[0][2] + $b,
        $a[0][3] + $b,
        $a[1][0] + $b,
        $a[1][1] + $b,
        $a[1][2] + $b,
        $a[1][3] + $b,
        $a[2][0] + $b,
        $a[2][1] + $b,
        $a[2][2] + $b,
        $a[2][3] + $b,
        $a[3][0] + $b,
        $a[3][1] + $b,
        $a[3][2] + $b,
        $a[3][3] + $b;
    $a;
}

multi sub infix:<+> (Mat44:D $a, Numeric:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] + $b,
        $a[0][1] + $b,
        $a[0][2] + $b,
        $a[0][3] + $b,
        $a[1][0] + $b,
        $a[1][1] + $b,
        $a[1][2] + $b,
        $a[1][3] + $b,
        $a[2][0] + $b,
        $a[2][1] + $b,
        $a[2][2] + $b,
        $a[2][3] + $b,
        $a[3][0] + $b,
        $a[3][1] + $b,
        $a[3][2] + $b,
        $a[3][3] + $b
    );
}

multi sub infix:<+> (Numeric:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a + $b[0][0],
        $a + $b[0][1],
        $a + $b[0][2],
        $a + $b[0][3],
        $a + $b[1][0],
        $a + $b[1][1],
        $a + $b[1][2],
        $a + $b[1][3],
        $a + $b[2][0],
        $a + $b[2][1],
        $a + $b[2][2],
        $a + $b[2][3],
        $a + $b[3][0],
        $a + $b[3][1],
        $a + $b[3][2],
        $a + $b[3][3]
    );
}

multi sub sub (Mat44:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] - $b[0][0],
        $a[0][1] - $b[0][1],
        $a[0][2] - $b[0][2],
        $a[0][3] - $b[0][3],
        $a[1][0] - $b[1][0],
        $a[1][1] - $b[1][1],
        $a[1][2] - $b[1][2],
        $a[1][3] - $b[1][3],
        $a[2][0] - $b[2][0],
        $a[2][1] - $b[2][1],
        $a[2][2] - $b[2][2],
        $a[2][3] - $b[2][3],
        $a[3][0] - $b[3][0],
        $a[3][1] - $b[3][1],
        $a[3][2] - $b[3][2],
        $a[3][3] - $b[3][3]
    );
}

multi method sub (Mat44:D $a: Mat44:D $b --> Mat44:D) is pure {
    Mat44.new(
        $a[0][0] - $b[0][0],
        $a[0][1] - $b[0][1],
        $a[0][2] - $b[0][2],
        $a[0][3] - $b[0][3],
        $a[1][0] - $b[1][0],
        $a[1][1] - $b[1][1],
        $a[1][2] - $b[1][2],
        $a[1][3] - $b[1][3],
        $a[2][0] - $b[2][0],
        $a[2][1] - $b[2][1],
        $a[2][2] - $b[2][2],
        $a[2][3] - $b[2][3],
        $a[3][0] - $b[3][0],
        $a[3][1] - $b[3][1],
        $a[3][2] - $b[3][2],
        $a[3][3] - $b[3][3]
    );
}

multi sub sub (Mat44:D $a, Mat44:D $b, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] - $b[0][0],
        $a[0][1] - $b[0][1],
        $a[0][2] - $b[0][2],
        $a[0][3] - $b[0][3],
        $a[1][0] - $b[1][0],
        $a[1][1] - $b[1][1],
        $a[1][2] - $b[1][2],
        $a[1][3] - $b[1][3],
        $a[2][0] - $b[2][0],
        $a[2][1] - $b[2][1],
        $a[2][2] - $b[2][2],
        $a[2][3] - $b[2][3],
        $a[3][0] - $b[3][0],
        $a[3][1] - $b[3][1],
        $a[3][2] - $b[3][2],
        $a[3][3] - $b[3][3];
    $r;
}

multi method sub (Mat44:D $a: Mat44:D $b, Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] - $b[0][0],
        $a[0][1] - $b[0][1],
        $a[0][2] - $b[0][2],
        $a[0][3] - $b[0][3],
        $a[1][0] - $b[1][0],
        $a[1][1] - $b[1][1],
        $a[1][2] - $b[1][2],
        $a[1][3] - $b[1][3],
        $a[2][0] - $b[2][0],
        $a[2][1] - $b[2][1],
        $a[2][2] - $b[2][2],
        $a[2][3] - $b[2][3],
        $a[3][0] - $b[3][0],
        $a[3][1] - $b[3][1],
        $a[3][2] - $b[3][2],
        $a[3][3] - $b[3][3];
    $r;
}

multi sub minus (Mat44:D $a, Mat44:D $b --> Mat44:D) is export {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] - $b[0][0],
        $a[0][1] - $b[0][1],
        $a[0][2] - $b[0][2],
        $a[0][3] - $b[0][3],
        $a[1][0] - $b[1][0],
        $a[1][1] - $b[1][1],
        $a[1][2] - $b[1][2],
        $a[1][3] - $b[1][3],
        $a[2][0] - $b[2][0],
        $a[2][1] - $b[2][1],
        $a[2][2] - $b[2][2],
        $a[2][3] - $b[2][3],
        $a[3][0] - $b[3][0],
        $a[3][1] - $b[3][1],
        $a[3][2] - $b[3][2],
        $a[3][3] - $b[3][3];
    $a;
}

multi method minus (Mat44:D $a: Mat44:D $b --> Mat44:D) {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] - $b[0][0],
        $a[0][1] - $b[0][1],
        $a[0][2] - $b[0][2],
        $a[0][3] - $b[0][3],
        $a[1][0] - $b[1][0],
        $a[1][1] - $b[1][1],
        $a[1][2] - $b[1][2],
        $a[1][3] - $b[1][3],
        $a[2][0] - $b[2][0],
        $a[2][1] - $b[2][1],
        $a[2][2] - $b[2][2],
        $a[2][3] - $b[2][3],
        $a[3][0] - $b[3][0],
        $a[3][1] - $b[3][1],
        $a[3][2] - $b[3][2],
        $a[3][3] - $b[3][3];
    $a;
}

multi sub infix:<-> (Mat44:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] - $b[0][0],
        $a[0][1] - $b[0][1],
        $a[0][2] - $b[0][2],
        $a[0][3] - $b[0][3],
        $a[1][0] - $b[1][0],
        $a[1][1] - $b[1][1],
        $a[1][2] - $b[1][2],
        $a[1][3] - $b[1][3],
        $a[2][0] - $b[2][0],
        $a[2][1] - $b[2][1],
        $a[2][2] - $b[2][2],
        $a[2][3] - $b[2][3],
        $a[3][0] - $b[3][0],
        $a[3][1] - $b[3][1],
        $a[3][2] - $b[3][2],
        $a[3][3] - $b[3][3]
    );
}

multi sub sub (Mat44:D $a, Numeric:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] - $b,
        $a[0][1] - $b,
        $a[0][2] - $b,
        $a[0][3] - $b,
        $a[1][0] - $b,
        $a[1][1] - $b,
        $a[1][2] - $b,
        $a[1][3] - $b,
        $a[2][0] - $b,
        $a[2][1] - $b,
        $a[2][2] - $b,
        $a[2][3] - $b,
        $a[3][0] - $b,
        $a[3][1] - $b,
        $a[3][2] - $b,
        $a[3][3] - $b
    );
}

multi method sub (Mat44:D $a: Numeric:D $b --> Mat44:D) is pure {
    Mat44.new(
        $a[0][0] - $b,
        $a[0][1] - $b,
        $a[0][2] - $b,
        $a[0][3] - $b,
        $a[1][0] - $b,
        $a[1][1] - $b,
        $a[1][2] - $b,
        $a[1][3] - $b,
        $a[2][0] - $b,
        $a[2][1] - $b,
        $a[2][2] - $b,
        $a[2][3] - $b,
        $a[3][0] - $b,
        $a[3][1] - $b,
        $a[3][2] - $b,
        $a[3][3] - $b
    );
}

multi sub sub (Mat44:D $a, Numeric:D $b, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] - $b,
        $a[0][1] - $b,
        $a[0][2] - $b,
        $a[0][3] - $b,
        $a[1][0] - $b,
        $a[1][1] - $b,
        $a[1][2] - $b,
        $a[1][3] - $b,
        $a[2][0] - $b,
        $a[2][1] - $b,
        $a[2][2] - $b,
        $a[2][3] - $b,
        $a[3][0] - $b,
        $a[3][1] - $b,
        $a[3][2] - $b,
        $a[3][3] - $b;
    $r;
}

multi method sub (Mat44:D $a: Numeric:D $b, Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] - $b,
        $a[0][1] - $b,
        $a[0][2] - $b,
        $a[0][3] - $b,
        $a[1][0] - $b,
        $a[1][1] - $b,
        $a[1][2] - $b,
        $a[1][3] - $b,
        $a[2][0] - $b,
        $a[2][1] - $b,
        $a[2][2] - $b,
        $a[2][3] - $b,
        $a[3][0] - $b,
        $a[3][1] - $b,
        $a[3][2] - $b,
        $a[3][3] - $b;
    $r;
}

multi sub minus (Mat44:D $a, Numeric:D $b --> Mat44:D) is export {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] - $b,
        $a[0][1] - $b,
        $a[0][2] - $b,
        $a[0][3] - $b,
        $a[1][0] - $b,
        $a[1][1] - $b,
        $a[1][2] - $b,
        $a[1][3] - $b,
        $a[2][0] - $b,
        $a[2][1] - $b,
        $a[2][2] - $b,
        $a[2][3] - $b,
        $a[3][0] - $b,
        $a[3][1] - $b,
        $a[3][2] - $b,
        $a[3][3] - $b;
    $a;
}

multi method minus (Mat44:D $a: Numeric:D $b --> Mat44:D) {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] - $b,
        $a[0][1] - $b,
        $a[0][2] - $b,
        $a[0][3] - $b,
        $a[1][0] - $b,
        $a[1][1] - $b,
        $a[1][2] - $b,
        $a[1][3] - $b,
        $a[2][0] - $b,
        $a[2][1] - $b,
        $a[2][2] - $b,
        $a[2][3] - $b,
        $a[3][0] - $b,
        $a[3][1] - $b,
        $a[3][2] - $b,
        $a[3][3] - $b;
    $a;
}

multi sub infix:<-> (Mat44:D $a, Numeric:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] - $b,
        $a[0][1] - $b,
        $a[0][2] - $b,
        $a[0][3] - $b,
        $a[1][0] - $b,
        $a[1][1] - $b,
        $a[1][2] - $b,
        $a[1][3] - $b,
        $a[2][0] - $b,
        $a[2][1] - $b,
        $a[2][2] - $b,
        $a[2][3] - $b,
        $a[3][0] - $b,
        $a[3][1] - $b,
        $a[3][2] - $b,
        $a[3][3] - $b
    );
}

multi sub infix:<-> (Numeric:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a - $b[0][0],
        $a - $b[0][1],
        $a - $b[0][2],
        $a - $b[0][3],
        $a - $b[1][0],
        $a - $b[1][1],
        $a - $b[1][2],
        $a - $b[1][3],
        $a - $b[2][0],
        $a - $b[2][1],
        $a - $b[2][2],
        $a - $b[2][3],
        $a - $b[3][0],
        $a - $b[3][1],
        $a - $b[3][2],
        $a - $b[3][3]
    );
}

multi sub neg (Mat44:D $a --> Mat44:D) is pure is export {
    Mat44.new(
        - $a[0][0],
        - $a[0][1],
        - $a[0][2],
        - $a[0][3],
        - $a[1][0],
        - $a[1][1],
        - $a[1][2],
        - $a[1][3],
        - $a[2][0],
        - $a[2][1],
        - $a[2][2],
        - $a[2][3],
        - $a[3][0],
        - $a[3][1],
        - $a[3][2],
        - $a[3][3]
    );
}

multi method neg (Mat44:D $a: --> Mat44:D) is pure {
    Mat44.new(
        - $a[0][0],
        - $a[0][1],
        - $a[0][2],
        - $a[0][3],
        - $a[1][0],
        - $a[1][1],
        - $a[1][2],
        - $a[1][3],
        - $a[2][0],
        - $a[2][1],
        - $a[2][2],
        - $a[2][3],
        - $a[3][0],
        - $a[3][1],
        - $a[3][2],
        - $a[3][3]
    );
}

multi sub neg (Mat44:D $a, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        - $a[0][0],
        - $a[0][1],
        - $a[0][2],
        - $a[0][3],
        - $a[1][0],
        - $a[1][1],
        - $a[1][2],
        - $a[1][3],
        - $a[2][0],
        - $a[2][1],
        - $a[2][2],
        - $a[2][3],
        - $a[3][0],
        - $a[3][1],
        - $a[3][2],
        - $a[3][3];
    $r;
}

multi method neg (Mat44:D $a: Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        - $a[0][0],
        - $a[0][1],
        - $a[0][2],
        - $a[0][3],
        - $a[1][0],
        - $a[1][1],
        - $a[1][2],
        - $a[1][3],
        - $a[2][0],
        - $a[2][1],
        - $a[2][2],
        - $a[2][3],
        - $a[3][0],
        - $a[3][1],
        - $a[3][2],
        - $a[3][3];
    $r;
}

multi sub negate (Mat44:D $a --> Mat44:D) is export {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        - $a[0][0],
        - $a[0][1],
        - $a[0][2],
        - $a[0][3],
        - $a[1][0],
        - $a[1][1],
        - $a[1][2],
        - $a[1][3],
        - $a[2][0],
        - $a[2][1],
        - $a[2][2],
        - $a[2][3],
        - $a[3][0],
        - $a[3][1],
        - $a[3][2],
        - $a[3][3];
    $a;
}

multi method negate (Mat44:D $a: --> Mat44:D) {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        - $a[0][0],
        - $a[0][1],
        - $a[0][2],
        - $a[0][3],
        - $a[1][0],
        - $a[1][1],
        - $a[1][2],
        - $a[1][3],
        - $a[2][0],
        - $a[2][1],
        - $a[2][2],
        - $a[2][3],
        - $a[3][0],
        - $a[3][1],
        - $a[3][2],
        - $a[3][3];
    $a;
}

multi sub prefix:<-> (Mat44:D $a --> Mat44:D) is pure is export {
    Mat44.new(
        - $a[0][0],
        - $a[0][1],
        - $a[0][2],
        - $a[0][3],
        - $a[1][0],
        - $a[1][1],
        - $a[1][2],
        - $a[1][3],
        - $a[2][0],
        - $a[2][1],
        - $a[2][2],
        - $a[2][3],
        - $a[3][0],
        - $a[3][1],
        - $a[3][2],
        - $a[3][3]
    );
}

multi sub mul (Mat44:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] * $b[0][0],
        $a[0][1] * $b[0][1],
        $a[0][2] * $b[0][2],
        $a[0][3] * $b[0][3],
        $a[1][0] * $b[1][0],
        $a[1][1] * $b[1][1],
        $a[1][2] * $b[1][2],
        $a[1][3] * $b[1][3],
        $a[2][0] * $b[2][0],
        $a[2][1] * $b[2][1],
        $a[2][2] * $b[2][2],
        $a[2][3] * $b[2][3],
        $a[3][0] * $b[3][0],
        $a[3][1] * $b[3][1],
        $a[3][2] * $b[3][2],
        $a[3][3] * $b[3][3]
    );
}

multi method mul (Mat44:D $a: Mat44:D $b --> Mat44:D) is pure {
    Mat44.new(
        $a[0][0] * $b[0][0],
        $a[0][1] * $b[0][1],
        $a[0][2] * $b[0][2],
        $a[0][3] * $b[0][3],
        $a[1][0] * $b[1][0],
        $a[1][1] * $b[1][1],
        $a[1][2] * $b[1][2],
        $a[1][3] * $b[1][3],
        $a[2][0] * $b[2][0],
        $a[2][1] * $b[2][1],
        $a[2][2] * $b[2][2],
        $a[2][3] * $b[2][3],
        $a[3][0] * $b[3][0],
        $a[3][1] * $b[3][1],
        $a[3][2] * $b[3][2],
        $a[3][3] * $b[3][3]
    );
}

multi sub mul (Mat44:D $a, Mat44:D $b, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] * $b[0][0],
        $a[0][1] * $b[0][1],
        $a[0][2] * $b[0][2],
        $a[0][3] * $b[0][3],
        $a[1][0] * $b[1][0],
        $a[1][1] * $b[1][1],
        $a[1][2] * $b[1][2],
        $a[1][3] * $b[1][3],
        $a[2][0] * $b[2][0],
        $a[2][1] * $b[2][1],
        $a[2][2] * $b[2][2],
        $a[2][3] * $b[2][3],
        $a[3][0] * $b[3][0],
        $a[3][1] * $b[3][1],
        $a[3][2] * $b[3][2],
        $a[3][3] * $b[3][3];
    $r;
}

multi method mul (Mat44:D $a: Mat44:D $b, Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] * $b[0][0],
        $a[0][1] * $b[0][1],
        $a[0][2] * $b[0][2],
        $a[0][3] * $b[0][3],
        $a[1][0] * $b[1][0],
        $a[1][1] * $b[1][1],
        $a[1][2] * $b[1][2],
        $a[1][3] * $b[1][3],
        $a[2][0] * $b[2][0],
        $a[2][1] * $b[2][1],
        $a[2][2] * $b[2][2],
        $a[2][3] * $b[2][3],
        $a[3][0] * $b[3][0],
        $a[3][1] * $b[3][1],
        $a[3][2] * $b[3][2],
        $a[3][3] * $b[3][3];
    $r;
}

multi sub times (Mat44:D $a, Mat44:D $b --> Mat44:D) is export {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] * $b[0][0],
        $a[0][1] * $b[0][1],
        $a[0][2] * $b[0][2],
        $a[0][3] * $b[0][3],
        $a[1][0] * $b[1][0],
        $a[1][1] * $b[1][1],
        $a[1][2] * $b[1][2],
        $a[1][3] * $b[1][3],
        $a[2][0] * $b[2][0],
        $a[2][1] * $b[2][1],
        $a[2][2] * $b[2][2],
        $a[2][3] * $b[2][3],
        $a[3][0] * $b[3][0],
        $a[3][1] * $b[3][1],
        $a[3][2] * $b[3][2],
        $a[3][3] * $b[3][3];
    $a;
}

multi method times (Mat44:D $a: Mat44:D $b --> Mat44:D) {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] * $b[0][0],
        $a[0][1] * $b[0][1],
        $a[0][2] * $b[0][2],
        $a[0][3] * $b[0][3],
        $a[1][0] * $b[1][0],
        $a[1][1] * $b[1][1],
        $a[1][2] * $b[1][2],
        $a[1][3] * $b[1][3],
        $a[2][0] * $b[2][0],
        $a[2][1] * $b[2][1],
        $a[2][2] * $b[2][2],
        $a[2][3] * $b[2][3],
        $a[3][0] * $b[3][0],
        $a[3][1] * $b[3][1],
        $a[3][2] * $b[3][2],
        $a[3][3] * $b[3][3];
    $a;
}

multi sub infix:<*> (Mat44:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] * $b[0][0],
        $a[0][1] * $b[0][1],
        $a[0][2] * $b[0][2],
        $a[0][3] * $b[0][3],
        $a[1][0] * $b[1][0],
        $a[1][1] * $b[1][1],
        $a[1][2] * $b[1][2],
        $a[1][3] * $b[1][3],
        $a[2][0] * $b[2][0],
        $a[2][1] * $b[2][1],
        $a[2][2] * $b[2][2],
        $a[2][3] * $b[2][3],
        $a[3][0] * $b[3][0],
        $a[3][1] * $b[3][1],
        $a[3][2] * $b[3][2],
        $a[3][3] * $b[3][3]
    );
}

multi sub mul (Mat44:D $a, Numeric:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] * $b,
        $a[0][1] * $b,
        $a[0][2] * $b,
        $a[0][3] * $b,
        $a[1][0] * $b,
        $a[1][1] * $b,
        $a[1][2] * $b,
        $a[1][3] * $b,
        $a[2][0] * $b,
        $a[2][1] * $b,
        $a[2][2] * $b,
        $a[2][3] * $b,
        $a[3][0] * $b,
        $a[3][1] * $b,
        $a[3][2] * $b,
        $a[3][3] * $b
    );
}

multi method mul (Mat44:D $a: Numeric:D $b --> Mat44:D) is pure {
    Mat44.new(
        $a[0][0] * $b,
        $a[0][1] * $b,
        $a[0][2] * $b,
        $a[0][3] * $b,
        $a[1][0] * $b,
        $a[1][1] * $b,
        $a[1][2] * $b,
        $a[1][3] * $b,
        $a[2][0] * $b,
        $a[2][1] * $b,
        $a[2][2] * $b,
        $a[2][3] * $b,
        $a[3][0] * $b,
        $a[3][1] * $b,
        $a[3][2] * $b,
        $a[3][3] * $b
    );
}

multi sub mul (Mat44:D $a, Numeric:D $b, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] * $b,
        $a[0][1] * $b,
        $a[0][2] * $b,
        $a[0][3] * $b,
        $a[1][0] * $b,
        $a[1][1] * $b,
        $a[1][2] * $b,
        $a[1][3] * $b,
        $a[2][0] * $b,
        $a[2][1] * $b,
        $a[2][2] * $b,
        $a[2][3] * $b,
        $a[3][0] * $b,
        $a[3][1] * $b,
        $a[3][2] * $b,
        $a[3][3] * $b;
    $r;
}

multi method mul (Mat44:D $a: Numeric:D $b, Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] * $b,
        $a[0][1] * $b,
        $a[0][2] * $b,
        $a[0][3] * $b,
        $a[1][0] * $b,
        $a[1][1] * $b,
        $a[1][2] * $b,
        $a[1][3] * $b,
        $a[2][0] * $b,
        $a[2][1] * $b,
        $a[2][2] * $b,
        $a[2][3] * $b,
        $a[3][0] * $b,
        $a[3][1] * $b,
        $a[3][2] * $b,
        $a[3][3] * $b;
    $r;
}

multi sub times (Mat44:D $a, Numeric:D $b --> Mat44:D) is export {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] * $b,
        $a[0][1] * $b,
        $a[0][2] * $b,
        $a[0][3] * $b,
        $a[1][0] * $b,
        $a[1][1] * $b,
        $a[1][2] * $b,
        $a[1][3] * $b,
        $a[2][0] * $b,
        $a[2][1] * $b,
        $a[2][2] * $b,
        $a[2][3] * $b,
        $a[3][0] * $b,
        $a[3][1] * $b,
        $a[3][2] * $b,
        $a[3][3] * $b;
    $a;
}

multi method times (Mat44:D $a: Numeric:D $b --> Mat44:D) {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] * $b,
        $a[0][1] * $b,
        $a[0][2] * $b,
        $a[0][3] * $b,
        $a[1][0] * $b,
        $a[1][1] * $b,
        $a[1][2] * $b,
        $a[1][3] * $b,
        $a[2][0] * $b,
        $a[2][1] * $b,
        $a[2][2] * $b,
        $a[2][3] * $b,
        $a[3][0] * $b,
        $a[3][1] * $b,
        $a[3][2] * $b,
        $a[3][3] * $b;
    $a;
}

multi sub infix:<*> (Mat44:D $a, Numeric:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] * $b,
        $a[0][1] * $b,
        $a[0][2] * $b,
        $a[0][3] * $b,
        $a[1][0] * $b,
        $a[1][1] * $b,
        $a[1][2] * $b,
        $a[1][3] * $b,
        $a[2][0] * $b,
        $a[2][1] * $b,
        $a[2][2] * $b,
        $a[2][3] * $b,
        $a[3][0] * $b,
        $a[3][1] * $b,
        $a[3][2] * $b,
        $a[3][3] * $b
    );
}

multi sub infix:<*> (Numeric:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a * $b[0][0],
        $a * $b[0][1],
        $a * $b[0][2],
        $a * $b[0][3],
        $a * $b[1][0],
        $a * $b[1][1],
        $a * $b[1][2],
        $a * $b[1][3],
        $a * $b[2][0],
        $a * $b[2][1],
        $a * $b[2][2],
        $a * $b[2][3],
        $a * $b[3][0],
        $a * $b[3][1],
        $a * $b[3][2],
        $a * $b[3][3]
    );
}

multi sub div (Mat44:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] / $b[0][0],
        $a[0][1] / $b[0][1],
        $a[0][2] / $b[0][2],
        $a[0][3] / $b[0][3],
        $a[1][0] / $b[1][0],
        $a[1][1] / $b[1][1],
        $a[1][2] / $b[1][2],
        $a[1][3] / $b[1][3],
        $a[2][0] / $b[2][0],
        $a[2][1] / $b[2][1],
        $a[2][2] / $b[2][2],
        $a[2][3] / $b[2][3],
        $a[3][0] / $b[3][0],
        $a[3][1] / $b[3][1],
        $a[3][2] / $b[3][2],
        $a[3][3] / $b[3][3]
    );
}

multi method div (Mat44:D $a: Mat44:D $b --> Mat44:D) is pure {
    Mat44.new(
        $a[0][0] / $b[0][0],
        $a[0][1] / $b[0][1],
        $a[0][2] / $b[0][2],
        $a[0][3] / $b[0][3],
        $a[1][0] / $b[1][0],
        $a[1][1] / $b[1][1],
        $a[1][2] / $b[1][2],
        $a[1][3] / $b[1][3],
        $a[2][0] / $b[2][0],
        $a[2][1] / $b[2][1],
        $a[2][2] / $b[2][2],
        $a[2][3] / $b[2][3],
        $a[3][0] / $b[3][0],
        $a[3][1] / $b[3][1],
        $a[3][2] / $b[3][2],
        $a[3][3] / $b[3][3]
    );
}

multi sub div (Mat44:D $a, Mat44:D $b, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] / $b[0][0],
        $a[0][1] / $b[0][1],
        $a[0][2] / $b[0][2],
        $a[0][3] / $b[0][3],
        $a[1][0] / $b[1][0],
        $a[1][1] / $b[1][1],
        $a[1][2] / $b[1][2],
        $a[1][3] / $b[1][3],
        $a[2][0] / $b[2][0],
        $a[2][1] / $b[2][1],
        $a[2][2] / $b[2][2],
        $a[2][3] / $b[2][3],
        $a[3][0] / $b[3][0],
        $a[3][1] / $b[3][1],
        $a[3][2] / $b[3][2],
        $a[3][3] / $b[3][3];
    $r;
}

multi method div (Mat44:D $a: Mat44:D $b, Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] / $b[0][0],
        $a[0][1] / $b[0][1],
        $a[0][2] / $b[0][2],
        $a[0][3] / $b[0][3],
        $a[1][0] / $b[1][0],
        $a[1][1] / $b[1][1],
        $a[1][2] / $b[1][2],
        $a[1][3] / $b[1][3],
        $a[2][0] / $b[2][0],
        $a[2][1] / $b[2][1],
        $a[2][2] / $b[2][2],
        $a[2][3] / $b[2][3],
        $a[3][0] / $b[3][0],
        $a[3][1] / $b[3][1],
        $a[3][2] / $b[3][2],
        $a[3][3] / $b[3][3];
    $r;
}

multi sub divide (Mat44:D $a, Mat44:D $b --> Mat44:D) is export {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] / $b[0][0],
        $a[0][1] / $b[0][1],
        $a[0][2] / $b[0][2],
        $a[0][3] / $b[0][3],
        $a[1][0] / $b[1][0],
        $a[1][1] / $b[1][1],
        $a[1][2] / $b[1][2],
        $a[1][3] / $b[1][3],
        $a[2][0] / $b[2][0],
        $a[2][1] / $b[2][1],
        $a[2][2] / $b[2][2],
        $a[2][3] / $b[2][3],
        $a[3][0] / $b[3][0],
        $a[3][1] / $b[3][1],
        $a[3][2] / $b[3][2],
        $a[3][3] / $b[3][3];
    $a;
}

multi method divide (Mat44:D $a: Mat44:D $b --> Mat44:D) {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] / $b[0][0],
        $a[0][1] / $b[0][1],
        $a[0][2] / $b[0][2],
        $a[0][3] / $b[0][3],
        $a[1][0] / $b[1][0],
        $a[1][1] / $b[1][1],
        $a[1][2] / $b[1][2],
        $a[1][3] / $b[1][3],
        $a[2][0] / $b[2][0],
        $a[2][1] / $b[2][1],
        $a[2][2] / $b[2][2],
        $a[2][3] / $b[2][3],
        $a[3][0] / $b[3][0],
        $a[3][1] / $b[3][1],
        $a[3][2] / $b[3][2],
        $a[3][3] / $b[3][3];
    $a;
}

multi sub infix:</> (Mat44:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] / $b[0][0],
        $a[0][1] / $b[0][1],
        $a[0][2] / $b[0][2],
        $a[0][3] / $b[0][3],
        $a[1][0] / $b[1][0],
        $a[1][1] / $b[1][1],
        $a[1][2] / $b[1][2],
        $a[1][3] / $b[1][3],
        $a[2][0] / $b[2][0],
        $a[2][1] / $b[2][1],
        $a[2][2] / $b[2][2],
        $a[2][3] / $b[2][3],
        $a[3][0] / $b[3][0],
        $a[3][1] / $b[3][1],
        $a[3][2] / $b[3][2],
        $a[3][3] / $b[3][3]
    );
}

multi sub div (Mat44:D $a, Numeric:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] / $b,
        $a[0][1] / $b,
        $a[0][2] / $b,
        $a[0][3] / $b,
        $a[1][0] / $b,
        $a[1][1] / $b,
        $a[1][2] / $b,
        $a[1][3] / $b,
        $a[2][0] / $b,
        $a[2][1] / $b,
        $a[2][2] / $b,
        $a[2][3] / $b,
        $a[3][0] / $b,
        $a[3][1] / $b,
        $a[3][2] / $b,
        $a[3][3] / $b
    );
}

multi method div (Mat44:D $a: Numeric:D $b --> Mat44:D) is pure {
    Mat44.new(
        $a[0][0] / $b,
        $a[0][1] / $b,
        $a[0][2] / $b,
        $a[0][3] / $b,
        $a[1][0] / $b,
        $a[1][1] / $b,
        $a[1][2] / $b,
        $a[1][3] / $b,
        $a[2][0] / $b,
        $a[2][1] / $b,
        $a[2][2] / $b,
        $a[2][3] / $b,
        $a[3][0] / $b,
        $a[3][1] / $b,
        $a[3][2] / $b,
        $a[3][3] / $b
    );
}

multi sub div (Mat44:D $a, Numeric:D $b, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] / $b,
        $a[0][1] / $b,
        $a[0][2] / $b,
        $a[0][3] / $b,
        $a[1][0] / $b,
        $a[1][1] / $b,
        $a[1][2] / $b,
        $a[1][3] / $b,
        $a[2][0] / $b,
        $a[2][1] / $b,
        $a[2][2] / $b,
        $a[2][3] / $b,
        $a[3][0] / $b,
        $a[3][1] / $b,
        $a[3][2] / $b,
        $a[3][3] / $b;
    $r;
}

multi method div (Mat44:D $a: Numeric:D $b, Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] / $b,
        $a[0][1] / $b,
        $a[0][2] / $b,
        $a[0][3] / $b,
        $a[1][0] / $b,
        $a[1][1] / $b,
        $a[1][2] / $b,
        $a[1][3] / $b,
        $a[2][0] / $b,
        $a[2][1] / $b,
        $a[2][2] / $b,
        $a[2][3] / $b,
        $a[3][0] / $b,
        $a[3][1] / $b,
        $a[3][2] / $b,
        $a[3][3] / $b;
    $r;
}

multi sub divide (Mat44:D $a, Numeric:D $b --> Mat44:D) is export {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] / $b,
        $a[0][1] / $b,
        $a[0][2] / $b,
        $a[0][3] / $b,
        $a[1][0] / $b,
        $a[1][1] / $b,
        $a[1][2] / $b,
        $a[1][3] / $b,
        $a[2][0] / $b,
        $a[2][1] / $b,
        $a[2][2] / $b,
        $a[2][3] / $b,
        $a[3][0] / $b,
        $a[3][1] / $b,
        $a[3][2] / $b,
        $a[3][3] / $b;
    $a;
}

multi method divide (Mat44:D $a: Numeric:D $b --> Mat44:D) {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] / $b,
        $a[0][1] / $b,
        $a[0][2] / $b,
        $a[0][3] / $b,
        $a[1][0] / $b,
        $a[1][1] / $b,
        $a[1][2] / $b,
        $a[1][3] / $b,
        $a[2][0] / $b,
        $a[2][1] / $b,
        $a[2][2] / $b,
        $a[2][3] / $b,
        $a[3][0] / $b,
        $a[3][1] / $b,
        $a[3][2] / $b,
        $a[3][3] / $b;
    $a;
}

multi sub infix:</> (Mat44:D $a, Numeric:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] / $b,
        $a[0][1] / $b,
        $a[0][2] / $b,
        $a[0][3] / $b,
        $a[1][0] / $b,
        $a[1][1] / $b,
        $a[1][2] / $b,
        $a[1][3] / $b,
        $a[2][0] / $b,
        $a[2][1] / $b,
        $a[2][2] / $b,
        $a[2][3] / $b,
        $a[3][0] / $b,
        $a[3][1] / $b,
        $a[3][2] / $b,
        $a[3][3] / $b
    );
}

multi sub infix:</> (Numeric:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a / $b[0][0],
        $a / $b[0][1],
        $a / $b[0][2],
        $a / $b[0][3],
        $a / $b[1][0],
        $a / $b[1][1],
        $a / $b[1][2],
        $a / $b[1][3],
        $a / $b[2][0],
        $a / $b[2][1],
        $a / $b[2][2],
        $a / $b[2][3],
        $a / $b[3][0],
        $a / $b[3][1],
        $a / $b[3][2],
        $a / $b[3][3]
    );
}

multi sub mod (Mat44:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] % $b[0][0],
        $a[0][1] % $b[0][1],
        $a[0][2] % $b[0][2],
        $a[0][3] % $b[0][3],
        $a[1][0] % $b[1][0],
        $a[1][1] % $b[1][1],
        $a[1][2] % $b[1][2],
        $a[1][3] % $b[1][3],
        $a[2][0] % $b[2][0],
        $a[2][1] % $b[2][1],
        $a[2][2] % $b[2][2],
        $a[2][3] % $b[2][3],
        $a[3][0] % $b[3][0],
        $a[3][1] % $b[3][1],
        $a[3][2] % $b[3][2],
        $a[3][3] % $b[3][3]
    );
}

multi method mod (Mat44:D $a: Mat44:D $b --> Mat44:D) is pure {
    Mat44.new(
        $a[0][0] % $b[0][0],
        $a[0][1] % $b[0][1],
        $a[0][2] % $b[0][2],
        $a[0][3] % $b[0][3],
        $a[1][0] % $b[1][0],
        $a[1][1] % $b[1][1],
        $a[1][2] % $b[1][2],
        $a[1][3] % $b[1][3],
        $a[2][0] % $b[2][0],
        $a[2][1] % $b[2][1],
        $a[2][2] % $b[2][2],
        $a[2][3] % $b[2][3],
        $a[3][0] % $b[3][0],
        $a[3][1] % $b[3][1],
        $a[3][2] % $b[3][2],
        $a[3][3] % $b[3][3]
    );
}

multi sub mod (Mat44:D $a, Mat44:D $b, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] % $b[0][0],
        $a[0][1] % $b[0][1],
        $a[0][2] % $b[0][2],
        $a[0][3] % $b[0][3],
        $a[1][0] % $b[1][0],
        $a[1][1] % $b[1][1],
        $a[1][2] % $b[1][2],
        $a[1][3] % $b[1][3],
        $a[2][0] % $b[2][0],
        $a[2][1] % $b[2][1],
        $a[2][2] % $b[2][2],
        $a[2][3] % $b[2][3],
        $a[3][0] % $b[3][0],
        $a[3][1] % $b[3][1],
        $a[3][2] % $b[3][2],
        $a[3][3] % $b[3][3];
    $r;
}

multi method mod (Mat44:D $a: Mat44:D $b, Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] % $b[0][0],
        $a[0][1] % $b[0][1],
        $a[0][2] % $b[0][2],
        $a[0][3] % $b[0][3],
        $a[1][0] % $b[1][0],
        $a[1][1] % $b[1][1],
        $a[1][2] % $b[1][2],
        $a[1][3] % $b[1][3],
        $a[2][0] % $b[2][0],
        $a[2][1] % $b[2][1],
        $a[2][2] % $b[2][2],
        $a[2][3] % $b[2][3],
        $a[3][0] % $b[3][0],
        $a[3][1] % $b[3][1],
        $a[3][2] % $b[3][2],
        $a[3][3] % $b[3][3];
    $r;
}

multi sub modulus (Mat44:D $a, Mat44:D $b --> Mat44:D) is export {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] % $b[0][0],
        $a[0][1] % $b[0][1],
        $a[0][2] % $b[0][2],
        $a[0][3] % $b[0][3],
        $a[1][0] % $b[1][0],
        $a[1][1] % $b[1][1],
        $a[1][2] % $b[1][2],
        $a[1][3] % $b[1][3],
        $a[2][0] % $b[2][0],
        $a[2][1] % $b[2][1],
        $a[2][2] % $b[2][2],
        $a[2][3] % $b[2][3],
        $a[3][0] % $b[3][0],
        $a[3][1] % $b[3][1],
        $a[3][2] % $b[3][2],
        $a[3][3] % $b[3][3];
    $a;
}

multi method modulus (Mat44:D $a: Mat44:D $b --> Mat44:D) {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] % $b[0][0],
        $a[0][1] % $b[0][1],
        $a[0][2] % $b[0][2],
        $a[0][3] % $b[0][3],
        $a[1][0] % $b[1][0],
        $a[1][1] % $b[1][1],
        $a[1][2] % $b[1][2],
        $a[1][3] % $b[1][3],
        $a[2][0] % $b[2][0],
        $a[2][1] % $b[2][1],
        $a[2][2] % $b[2][2],
        $a[2][3] % $b[2][3],
        $a[3][0] % $b[3][0],
        $a[3][1] % $b[3][1],
        $a[3][2] % $b[3][2],
        $a[3][3] % $b[3][3];
    $a;
}

multi sub infix:<%> (Mat44:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] % $b[0][0],
        $a[0][1] % $b[0][1],
        $a[0][2] % $b[0][2],
        $a[0][3] % $b[0][3],
        $a[1][0] % $b[1][0],
        $a[1][1] % $b[1][1],
        $a[1][2] % $b[1][2],
        $a[1][3] % $b[1][3],
        $a[2][0] % $b[2][0],
        $a[2][1] % $b[2][1],
        $a[2][2] % $b[2][2],
        $a[2][3] % $b[2][3],
        $a[3][0] % $b[3][0],
        $a[3][1] % $b[3][1],
        $a[3][2] % $b[3][2],
        $a[3][3] % $b[3][3]
    );
}

multi sub mod (Mat44:D $a, Numeric:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] % $b,
        $a[0][1] % $b,
        $a[0][2] % $b,
        $a[0][3] % $b,
        $a[1][0] % $b,
        $a[1][1] % $b,
        $a[1][2] % $b,
        $a[1][3] % $b,
        $a[2][0] % $b,
        $a[2][1] % $b,
        $a[2][2] % $b,
        $a[2][3] % $b,
        $a[3][0] % $b,
        $a[3][1] % $b,
        $a[3][2] % $b,
        $a[3][3] % $b
    );
}

multi method mod (Mat44:D $a: Numeric:D $b --> Mat44:D) is pure {
    Mat44.new(
        $a[0][0] % $b,
        $a[0][1] % $b,
        $a[0][2] % $b,
        $a[0][3] % $b,
        $a[1][0] % $b,
        $a[1][1] % $b,
        $a[1][2] % $b,
        $a[1][3] % $b,
        $a[2][0] % $b,
        $a[2][1] % $b,
        $a[2][2] % $b,
        $a[2][3] % $b,
        $a[3][0] % $b,
        $a[3][1] % $b,
        $a[3][2] % $b,
        $a[3][3] % $b
    );
}

multi sub mod (Mat44:D $a, Numeric:D $b, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] % $b,
        $a[0][1] % $b,
        $a[0][2] % $b,
        $a[0][3] % $b,
        $a[1][0] % $b,
        $a[1][1] % $b,
        $a[1][2] % $b,
        $a[1][3] % $b,
        $a[2][0] % $b,
        $a[2][1] % $b,
        $a[2][2] % $b,
        $a[2][3] % $b,
        $a[3][0] % $b,
        $a[3][1] % $b,
        $a[3][2] % $b,
        $a[3][3] % $b;
    $r;
}

multi method mod (Mat44:D $a: Numeric:D $b, Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0] % $b,
        $a[0][1] % $b,
        $a[0][2] % $b,
        $a[0][3] % $b,
        $a[1][0] % $b,
        $a[1][1] % $b,
        $a[1][2] % $b,
        $a[1][3] % $b,
        $a[2][0] % $b,
        $a[2][1] % $b,
        $a[2][2] % $b,
        $a[2][3] % $b,
        $a[3][0] % $b,
        $a[3][1] % $b,
        $a[3][2] % $b,
        $a[3][3] % $b;
    $r;
}

multi sub modulus (Mat44:D $a, Numeric:D $b --> Mat44:D) is export {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] % $b,
        $a[0][1] % $b,
        $a[0][2] % $b,
        $a[0][3] % $b,
        $a[1][0] % $b,
        $a[1][1] % $b,
        $a[1][2] % $b,
        $a[1][3] % $b,
        $a[2][0] % $b,
        $a[2][1] % $b,
        $a[2][2] % $b,
        $a[2][3] % $b,
        $a[3][0] % $b,
        $a[3][1] % $b,
        $a[3][2] % $b,
        $a[3][3] % $b;
    $a;
}

multi method modulus (Mat44:D $a: Numeric:D $b --> Mat44:D) {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0] % $b,
        $a[0][1] % $b,
        $a[0][2] % $b,
        $a[0][3] % $b,
        $a[1][0] % $b,
        $a[1][1] % $b,
        $a[1][2] % $b,
        $a[1][3] % $b,
        $a[2][0] % $b,
        $a[2][1] % $b,
        $a[2][2] % $b,
        $a[2][3] % $b,
        $a[3][0] % $b,
        $a[3][1] % $b,
        $a[3][2] % $b,
        $a[3][3] % $b;
    $a;
}

multi sub infix:<%> (Mat44:D $a, Numeric:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0] % $b,
        $a[0][1] % $b,
        $a[0][2] % $b,
        $a[0][3] % $b,
        $a[1][0] % $b,
        $a[1][1] % $b,
        $a[1][2] % $b,
        $a[1][3] % $b,
        $a[2][0] % $b,
        $a[2][1] % $b,
        $a[2][2] % $b,
        $a[2][3] % $b,
        $a[3][0] % $b,
        $a[3][1] % $b,
        $a[3][2] % $b,
        $a[3][3] % $b
    );
}

multi sub infix:<%> (Numeric:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a % $b[0][0],
        $a % $b[0][1],
        $a % $b[0][2],
        $a % $b[0][3],
        $a % $b[1][0],
        $a % $b[1][1],
        $a % $b[1][2],
        $a % $b[1][3],
        $a % $b[2][0],
        $a % $b[2][1],
        $a % $b[2][2],
        $a % $b[2][3],
        $a % $b[3][0],
        $a % $b[3][1],
        $a % $b[3][2],
        $a % $b[3][3]
    );
}

multi sub inv (Mat44:D $a --> Mat44:D) is pure is export {
    my $s = [
        $a[0][0] * $a[1][1] - $a[1][0] * $a[0][1],
        $a[0][0] * $a[1][2] - $a[1][0] * $a[0][2],
        $a[0][0] * $a[1][3] - $a[1][0] * $a[0][3],
        $a[0][1] * $a[1][2] - $a[1][1] * $a[0][2],
        $a[0][1] * $a[1][3] - $a[1][1] * $a[0][3],
        $a[0][2] * $a[1][3] - $a[1][2] * $a[0][3]
    ];

    my $c = [
        $a[2][0] * $a[3][1] - $a[3][0] * $a[2][1],
        $a[2][0] * $a[3][2] - $a[3][0] * $a[2][2],
        $a[2][0] * $a[3][3] - $a[3][0] * $a[2][3],
        $a[2][1] * $a[3][2] - $a[3][1] * $a[2][2],
        $a[2][1] * $a[3][3] - $a[3][1] * $a[2][3],
        $a[2][2] * $a[3][3] - $a[3][2] * $a[2][3]
    ];

    my $det =
        $s[0] * $c[5] -
        $s[1] * $c[4] +
        $s[2] * $c[3] +
        $s[3] * $c[2] -
        $s[4] * $c[1] +
        $s[5] * $c[0];

    die "Cannot invert zero-determinant matrix:\n{$a.perl}" unless $det;

    my $invdet = 1 / $det;

    Mat44.new(
        ( $a[1][1] * $c[5] - $a[1][2] * $c[4] + $a[1][3] * $c[3]) * $invdet,
        (-$a[0][1] * $c[5] + $a[0][2] * $c[4] - $a[0][3] * $c[3]) * $invdet,
        ( $a[3][1] * $s[5] - $a[3][2] * $s[4] + $a[3][3] * $s[3]) * $invdet,
        (-$a[2][1] * $s[5] + $a[2][2] * $s[4] - $a[2][3] * $s[3]) * $invdet,
        (-$a[1][0] * $c[5] + $a[1][2] * $c[2] - $a[1][3] * $c[1]) * $invdet,
        ( $a[0][0] * $c[5] - $a[0][2] * $c[2] + $a[0][3] * $c[1]) * $invdet,
        (-$a[3][0] * $s[5] + $a[3][2] * $s[2] - $a[3][3] * $s[1]) * $invdet,
        ( $a[2][0] * $s[5] - $a[2][2] * $s[2] + $a[2][3] * $s[1]) * $invdet,
        ( $a[1][0] * $c[4] - $a[1][1] * $c[2] + $a[1][3] * $c[0]) * $invdet,
        (-$a[0][0] * $c[4] + $a[0][1] * $c[2] - $a[0][3] * $c[0]) * $invdet,
        ( $a[3][0] * $s[4] - $a[3][1] * $s[2] + $a[3][3] * $s[0]) * $invdet,
        (-$a[2][0] * $s[4] + $a[2][1] * $s[2] - $a[2][3] * $s[0]) * $invdet,
        (-$a[1][0] * $c[3] + $a[1][1] * $c[1] - $a[1][2] * $c[0]) * $invdet,
        ( $a[0][0] * $c[3] - $a[0][1] * $c[1] + $a[0][2] * $c[0]) * $invdet,
        (-$a[3][0] * $s[3] + $a[3][1] * $s[1] - $a[3][2] * $s[0]) * $invdet,
        ( $a[2][0] * $s[3] - $a[2][1] * $s[1] + $a[2][2] * $s[0]) * $invdet
    );
}

multi method inv (Mat44:D $a: --> Mat44:D) is pure {
    my $s = [
        $a[0][0] * $a[1][1] - $a[1][0] * $a[0][1],
        $a[0][0] * $a[1][2] - $a[1][0] * $a[0][2],
        $a[0][0] * $a[1][3] - $a[1][0] * $a[0][3],
        $a[0][1] * $a[1][2] - $a[1][1] * $a[0][2],
        $a[0][1] * $a[1][3] - $a[1][1] * $a[0][3],
        $a[0][2] * $a[1][3] - $a[1][2] * $a[0][3]
    ];

    my $c = [
        $a[2][0] * $a[3][1] - $a[3][0] * $a[2][1],
        $a[2][0] * $a[3][2] - $a[3][0] * $a[2][2],
        $a[2][0] * $a[3][3] - $a[3][0] * $a[2][3],
        $a[2][1] * $a[3][2] - $a[3][1] * $a[2][2],
        $a[2][1] * $a[3][3] - $a[3][1] * $a[2][3],
        $a[2][2] * $a[3][3] - $a[3][2] * $a[2][3]
    ];

    my $det =
        $s[0] * $c[5] -
        $s[1] * $c[4] +
        $s[2] * $c[3] +
        $s[3] * $c[2] -
        $s[4] * $c[1] +
        $s[5] * $c[0];

    die "Cannot invert zero-determinant matrix:\n{$a.perl}" unless $det;

    my $invdet = 1 / $det;

    Mat44.new(
        ( $a[1][1] * $c[5] - $a[1][2] * $c[4] + $a[1][3] * $c[3]) * $invdet,
        (-$a[0][1] * $c[5] + $a[0][2] * $c[4] - $a[0][3] * $c[3]) * $invdet,
        ( $a[3][1] * $s[5] - $a[3][2] * $s[4] + $a[3][3] * $s[3]) * $invdet,
        (-$a[2][1] * $s[5] + $a[2][2] * $s[4] - $a[2][3] * $s[3]) * $invdet,
        (-$a[1][0] * $c[5] + $a[1][2] * $c[2] - $a[1][3] * $c[1]) * $invdet,
        ( $a[0][0] * $c[5] - $a[0][2] * $c[2] + $a[0][3] * $c[1]) * $invdet,
        (-$a[3][0] * $s[5] + $a[3][2] * $s[2] - $a[3][3] * $s[1]) * $invdet,
        ( $a[2][0] * $s[5] - $a[2][2] * $s[2] + $a[2][3] * $s[1]) * $invdet,
        ( $a[1][0] * $c[4] - $a[1][1] * $c[2] + $a[1][3] * $c[0]) * $invdet,
        (-$a[0][0] * $c[4] + $a[0][1] * $c[2] - $a[0][3] * $c[0]) * $invdet,
        ( $a[3][0] * $s[4] - $a[3][1] * $s[2] + $a[3][3] * $s[0]) * $invdet,
        (-$a[2][0] * $s[4] + $a[2][1] * $s[2] - $a[2][3] * $s[0]) * $invdet,
        (-$a[1][0] * $c[3] + $a[1][1] * $c[1] - $a[1][2] * $c[0]) * $invdet,
        ( $a[0][0] * $c[3] - $a[0][1] * $c[1] + $a[0][2] * $c[0]) * $invdet,
        (-$a[3][0] * $s[3] + $a[3][1] * $s[1] - $a[3][2] * $s[0]) * $invdet,
        ( $a[2][0] * $s[3] - $a[2][1] * $s[1] + $a[2][2] * $s[0]) * $invdet
    );
}

multi sub inv (Mat44:D $a, Mat44:D $r is rw --> Mat44:D) is export {
    my $s = [
        $a[0][0] * $a[1][1] - $a[1][0] * $a[0][1],
        $a[0][0] * $a[1][2] - $a[1][0] * $a[0][2],
        $a[0][0] * $a[1][3] - $a[1][0] * $a[0][3],
        $a[0][1] * $a[1][2] - $a[1][1] * $a[0][2],
        $a[0][1] * $a[1][3] - $a[1][1] * $a[0][3],
        $a[0][2] * $a[1][3] - $a[1][2] * $a[0][3]
    ];

    my $c = [
        $a[2][0] * $a[3][1] - $a[3][0] * $a[2][1],
        $a[2][0] * $a[3][2] - $a[3][0] * $a[2][2],
        $a[2][0] * $a[3][3] - $a[3][0] * $a[2][3],
        $a[2][1] * $a[3][2] - $a[3][1] * $a[2][2],
        $a[2][1] * $a[3][3] - $a[3][1] * $a[2][3],
        $a[2][2] * $a[3][3] - $a[3][2] * $a[2][3]
    ];

    my $det =
        $s[0] * $c[5] -
        $s[1] * $c[4] +
        $s[2] * $c[3] +
        $s[3] * $c[2] -
        $s[4] * $c[1] +
        $s[5] * $c[0];

    die "Cannot invert zero-determinant matrix:\n{$a.perl}" unless $det;

    my $invdet = 1 / $det;

    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        ( $a[1][1] * $c[5] - $a[1][2] * $c[4] + $a[1][3] * $c[3]) * $invdet,
        (-$a[0][1] * $c[5] + $a[0][2] * $c[4] - $a[0][3] * $c[3]) * $invdet,
        ( $a[3][1] * $s[5] - $a[3][2] * $s[4] + $a[3][3] * $s[3]) * $invdet,
        (-$a[2][1] * $s[5] + $a[2][2] * $s[4] - $a[2][3] * $s[3]) * $invdet,
        (-$a[1][0] * $c[5] + $a[1][2] * $c[2] - $a[1][3] * $c[1]) * $invdet,
        ( $a[0][0] * $c[5] - $a[0][2] * $c[2] + $a[0][3] * $c[1]) * $invdet,
        (-$a[3][0] * $s[5] + $a[3][2] * $s[2] - $a[3][3] * $s[1]) * $invdet,
        ( $a[2][0] * $s[5] - $a[2][2] * $s[2] + $a[2][3] * $s[1]) * $invdet,
        ( $a[1][0] * $c[4] - $a[1][1] * $c[2] + $a[1][3] * $c[0]) * $invdet,
        (-$a[0][0] * $c[4] + $a[0][1] * $c[2] - $a[0][3] * $c[0]) * $invdet,
        ( $a[3][0] * $s[4] - $a[3][1] * $s[2] + $a[3][3] * $s[0]) * $invdet,
        (-$a[2][0] * $s[4] + $a[2][1] * $s[2] - $a[2][3] * $s[0]) * $invdet,
        (-$a[1][0] * $c[3] + $a[1][1] * $c[1] - $a[1][2] * $c[0]) * $invdet,
        ( $a[0][0] * $c[3] - $a[0][1] * $c[1] + $a[0][2] * $c[0]) * $invdet,
        (-$a[3][0] * $s[3] + $a[3][1] * $s[1] - $a[3][2] * $s[0]) * $invdet,
        ( $a[2][0] * $s[3] - $a[2][1] * $s[1] + $a[2][2] * $s[0]) * $invdet;
    $r;
}

multi method inv (Mat44:D $a: Mat44:D $r is rw --> Mat44:D) {
    my $s = [
        $a[0][0] * $a[1][1] - $a[1][0] * $a[0][1],
        $a[0][0] * $a[1][2] - $a[1][0] * $a[0][2],
        $a[0][0] * $a[1][3] - $a[1][0] * $a[0][3],
        $a[0][1] * $a[1][2] - $a[1][1] * $a[0][2],
        $a[0][1] * $a[1][3] - $a[1][1] * $a[0][3],
        $a[0][2] * $a[1][3] - $a[1][2] * $a[0][3]
    ];

    my $c = [
        $a[2][0] * $a[3][1] - $a[3][0] * $a[2][1],
        $a[2][0] * $a[3][2] - $a[3][0] * $a[2][2],
        $a[2][0] * $a[3][3] - $a[3][0] * $a[2][3],
        $a[2][1] * $a[3][2] - $a[3][1] * $a[2][2],
        $a[2][1] * $a[3][3] - $a[3][1] * $a[2][3],
        $a[2][2] * $a[3][3] - $a[3][2] * $a[2][3]
    ];

    my $det =
        $s[0] * $c[5] -
        $s[1] * $c[4] +
        $s[2] * $c[3] +
        $s[3] * $c[2] -
        $s[4] * $c[1] +
        $s[5] * $c[0];

    die "Cannot invert zero-determinant matrix:\n{$a.perl}" unless $det;

    my $invdet = 1 / $det;

    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        ( $a[1][1] * $c[5] - $a[1][2] * $c[4] + $a[1][3] * $c[3]) * $invdet,
        (-$a[0][1] * $c[5] + $a[0][2] * $c[4] - $a[0][3] * $c[3]) * $invdet,
        ( $a[3][1] * $s[5] - $a[3][2] * $s[4] + $a[3][3] * $s[3]) * $invdet,
        (-$a[2][1] * $s[5] + $a[2][2] * $s[4] - $a[2][3] * $s[3]) * $invdet,
        (-$a[1][0] * $c[5] + $a[1][2] * $c[2] - $a[1][3] * $c[1]) * $invdet,
        ( $a[0][0] * $c[5] - $a[0][2] * $c[2] + $a[0][3] * $c[1]) * $invdet,
        (-$a[3][0] * $s[5] + $a[3][2] * $s[2] - $a[3][3] * $s[1]) * $invdet,
        ( $a[2][0] * $s[5] - $a[2][2] * $s[2] + $a[2][3] * $s[1]) * $invdet,
        ( $a[1][0] * $c[4] - $a[1][1] * $c[2] + $a[1][3] * $c[0]) * $invdet,
        (-$a[0][0] * $c[4] + $a[0][1] * $c[2] - $a[0][3] * $c[0]) * $invdet,
        ( $a[3][0] * $s[4] - $a[3][1] * $s[2] + $a[3][3] * $s[0]) * $invdet,
        (-$a[2][0] * $s[4] + $a[2][1] * $s[2] - $a[2][3] * $s[0]) * $invdet,
        (-$a[1][0] * $c[3] + $a[1][1] * $c[1] - $a[1][2] * $c[0]) * $invdet,
        ( $a[0][0] * $c[3] - $a[0][1] * $c[1] + $a[0][2] * $c[0]) * $invdet,
        (-$a[3][0] * $s[3] + $a[3][1] * $s[1] - $a[3][2] * $s[0]) * $invdet,
        ( $a[2][0] * $s[3] - $a[2][1] * $s[1] + $a[2][2] * $s[0]) * $invdet;
    $r;
}

multi sub invert (Mat44:D $a --> Mat44:D) is export {
    my $s = [
        $a[0][0] * $a[1][1] - $a[1][0] * $a[0][1],
        $a[0][0] * $a[1][2] - $a[1][0] * $a[0][2],
        $a[0][0] * $a[1][3] - $a[1][0] * $a[0][3],
        $a[0][1] * $a[1][2] - $a[1][1] * $a[0][2],
        $a[0][1] * $a[1][3] - $a[1][1] * $a[0][3],
        $a[0][2] * $a[1][3] - $a[1][2] * $a[0][3]
    ];

    my $c = [
        $a[2][0] * $a[3][1] - $a[3][0] * $a[2][1],
        $a[2][0] * $a[3][2] - $a[3][0] * $a[2][2],
        $a[2][0] * $a[3][3] - $a[3][0] * $a[2][3],
        $a[2][1] * $a[3][2] - $a[3][1] * $a[2][2],
        $a[2][1] * $a[3][3] - $a[3][1] * $a[2][3],
        $a[2][2] * $a[3][3] - $a[3][2] * $a[2][3]
    ];

    my $det =
        $s[0] * $c[5] -
        $s[1] * $c[4] +
        $s[2] * $c[3] +
        $s[3] * $c[2] -
        $s[4] * $c[1] +
        $s[5] * $c[0];

    die "Cannot invert zero-determinant matrix:\n{$a.perl}" unless $det;

    my $invdet = 1 / $det;

    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        ( $a[1][1] * $c[5] - $a[1][2] * $c[4] + $a[1][3] * $c[3]) * $invdet,
        (-$a[0][1] * $c[5] + $a[0][2] * $c[4] - $a[0][3] * $c[3]) * $invdet,
        ( $a[3][1] * $s[5] - $a[3][2] * $s[4] + $a[3][3] * $s[3]) * $invdet,
        (-$a[2][1] * $s[5] + $a[2][2] * $s[4] - $a[2][3] * $s[3]) * $invdet,
        (-$a[1][0] * $c[5] + $a[1][2] * $c[2] - $a[1][3] * $c[1]) * $invdet,
        ( $a[0][0] * $c[5] - $a[0][2] * $c[2] + $a[0][3] * $c[1]) * $invdet,
        (-$a[3][0] * $s[5] + $a[3][2] * $s[2] - $a[3][3] * $s[1]) * $invdet,
        ( $a[2][0] * $s[5] - $a[2][2] * $s[2] + $a[2][3] * $s[1]) * $invdet,
        ( $a[1][0] * $c[4] - $a[1][1] * $c[2] + $a[1][3] * $c[0]) * $invdet,
        (-$a[0][0] * $c[4] + $a[0][1] * $c[2] - $a[0][3] * $c[0]) * $invdet,
        ( $a[3][0] * $s[4] - $a[3][1] * $s[2] + $a[3][3] * $s[0]) * $invdet,
        (-$a[2][0] * $s[4] + $a[2][1] * $s[2] - $a[2][3] * $s[0]) * $invdet,
        (-$a[1][0] * $c[3] + $a[1][1] * $c[1] - $a[1][2] * $c[0]) * $invdet,
        ( $a[0][0] * $c[3] - $a[0][1] * $c[1] + $a[0][2] * $c[0]) * $invdet,
        (-$a[3][0] * $s[3] + $a[3][1] * $s[1] - $a[3][2] * $s[0]) * $invdet,
        ( $a[2][0] * $s[3] - $a[2][1] * $s[1] + $a[2][2] * $s[0]) * $invdet;
    $a;
}

multi method invert (Mat44:D $a: --> Mat44:D) {
    my $s = [
        $a[0][0] * $a[1][1] - $a[1][0] * $a[0][1],
        $a[0][0] * $a[1][2] - $a[1][0] * $a[0][2],
        $a[0][0] * $a[1][3] - $a[1][0] * $a[0][3],
        $a[0][1] * $a[1][2] - $a[1][1] * $a[0][2],
        $a[0][1] * $a[1][3] - $a[1][1] * $a[0][3],
        $a[0][2] * $a[1][3] - $a[1][2] * $a[0][3]
    ];

    my $c = [
        $a[2][0] * $a[3][1] - $a[3][0] * $a[2][1],
        $a[2][0] * $a[3][2] - $a[3][0] * $a[2][2],
        $a[2][0] * $a[3][3] - $a[3][0] * $a[2][3],
        $a[2][1] * $a[3][2] - $a[3][1] * $a[2][2],
        $a[2][1] * $a[3][3] - $a[3][1] * $a[2][3],
        $a[2][2] * $a[3][3] - $a[3][2] * $a[2][3]
    ];

    my $det =
        $s[0] * $c[5] -
        $s[1] * $c[4] +
        $s[2] * $c[3] +
        $s[3] * $c[2] -
        $s[4] * $c[1] +
        $s[5] * $c[0];

    die "Cannot invert zero-determinant matrix:\n{$a.perl}" unless $det;

    my $invdet = 1 / $det;

    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        ( $a[1][1] * $c[5] - $a[1][2] * $c[4] + $a[1][3] * $c[3]) * $invdet,
        (-$a[0][1] * $c[5] + $a[0][2] * $c[4] - $a[0][3] * $c[3]) * $invdet,
        ( $a[3][1] * $s[5] - $a[3][2] * $s[4] + $a[3][3] * $s[3]) * $invdet,
        (-$a[2][1] * $s[5] + $a[2][2] * $s[4] - $a[2][3] * $s[3]) * $invdet,
        (-$a[1][0] * $c[5] + $a[1][2] * $c[2] - $a[1][3] * $c[1]) * $invdet,
        ( $a[0][0] * $c[5] - $a[0][2] * $c[2] + $a[0][3] * $c[1]) * $invdet,
        (-$a[3][0] * $s[5] + $a[3][2] * $s[2] - $a[3][3] * $s[1]) * $invdet,
        ( $a[2][0] * $s[5] - $a[2][2] * $s[2] + $a[2][3] * $s[1]) * $invdet,
        ( $a[1][0] * $c[4] - $a[1][1] * $c[2] + $a[1][3] * $c[0]) * $invdet,
        (-$a[0][0] * $c[4] + $a[0][1] * $c[2] - $a[0][3] * $c[0]) * $invdet,
        ( $a[3][0] * $s[4] - $a[3][1] * $s[2] + $a[3][3] * $s[0]) * $invdet,
        (-$a[2][0] * $s[4] + $a[2][1] * $s[2] - $a[2][3] * $s[0]) * $invdet,
        (-$a[1][0] * $c[3] + $a[1][1] * $c[1] - $a[1][2] * $c[0]) * $invdet,
        ( $a[0][0] * $c[3] - $a[0][1] * $c[1] + $a[0][2] * $c[0]) * $invdet,
        (-$a[3][0] * $s[3] + $a[3][1] * $s[1] - $a[3][2] * $s[0]) * $invdet,
        ( $a[2][0] * $s[3] - $a[2][1] * $s[1] + $a[2][2] * $s[0]) * $invdet;
    $a;
}

multi sub prod (Mat44:D $a, Mat44:D $b --> Mat44:D) is pure is export {
    Mat44.new(
        $a[0][0]*$b[0][0] + $a[0][1]*$b[1][0] + $a[0][2]*$b[2][0] + $a[0][3]*$b[3][0],
        $a[0][0]*$b[0][1] + $a[0][1]*$b[1][1] + $a[0][2]*$b[2][1] + $a[0][3]*$b[3][1],
        $a[0][0]*$b[0][2] + $a[0][1]*$b[1][2] + $a[0][2]*$b[2][2] + $a[0][3]*$b[3][2],
        $a[0][0]*$b[0][3] + $a[0][1]*$b[1][3] + $a[0][2]*$b[2][3] + $a[0][3]*$b[3][3],
        $a[1][0]*$b[0][0] + $a[1][1]*$b[1][0] + $a[1][2]*$b[2][0] + $a[1][3]*$b[3][0],
        $a[1][0]*$b[0][1] + $a[1][1]*$b[1][1] + $a[1][2]*$b[2][1] + $a[1][3]*$b[3][1],
        $a[1][0]*$b[0][2] + $a[1][1]*$b[1][2] + $a[1][2]*$b[2][2] + $a[1][3]*$b[3][2],
        $a[1][0]*$b[0][3] + $a[1][1]*$b[1][3] + $a[1][2]*$b[2][3] + $a[1][3]*$b[3][3],
        $a[2][0]*$b[0][0] + $a[2][1]*$b[1][0] + $a[2][2]*$b[2][0] + $a[2][3]*$b[3][0],
        $a[2][0]*$b[0][1] + $a[2][1]*$b[1][1] + $a[2][2]*$b[2][1] + $a[2][3]*$b[3][1],
        $a[2][0]*$b[0][2] + $a[2][1]*$b[1][2] + $a[2][2]*$b[2][2] + $a[2][3]*$b[3][2],
        $a[2][0]*$b[0][3] + $a[2][1]*$b[1][3] + $a[2][2]*$b[2][3] + $a[2][3]*$b[3][3],
        $a[3][0]*$b[0][0] + $a[3][1]*$b[1][0] + $a[3][2]*$b[2][0] + $a[3][3]*$b[3][0],
        $a[3][0]*$b[0][1] + $a[3][1]*$b[1][1] + $a[3][2]*$b[2][1] + $a[3][3]*$b[3][1],
        $a[3][0]*$b[0][2] + $a[3][1]*$b[1][2] + $a[3][2]*$b[2][2] + $a[3][3]*$b[3][2],
        $a[3][0]*$b[0][3] + $a[3][1]*$b[1][3] + $a[3][2]*$b[2][3] + $a[3][3]*$b[3][3]
    );
}

multi method prod (Mat44:D $a: Mat44:D $b --> Mat44:D) is pure {
    Mat44.new(
        $a[0][0]*$b[0][0] + $a[0][1]*$b[1][0] + $a[0][2]*$b[2][0] + $a[0][3]*$b[3][0],
        $a[0][0]*$b[0][1] + $a[0][1]*$b[1][1] + $a[0][2]*$b[2][1] + $a[0][3]*$b[3][1],
        $a[0][0]*$b[0][2] + $a[0][1]*$b[1][2] + $a[0][2]*$b[2][2] + $a[0][3]*$b[3][2],
        $a[0][0]*$b[0][3] + $a[0][1]*$b[1][3] + $a[0][2]*$b[2][3] + $a[0][3]*$b[3][3],
        $a[1][0]*$b[0][0] + $a[1][1]*$b[1][0] + $a[1][2]*$b[2][0] + $a[1][3]*$b[3][0],
        $a[1][0]*$b[0][1] + $a[1][1]*$b[1][1] + $a[1][2]*$b[2][1] + $a[1][3]*$b[3][1],
        $a[1][0]*$b[0][2] + $a[1][1]*$b[1][2] + $a[1][2]*$b[2][2] + $a[1][3]*$b[3][2],
        $a[1][0]*$b[0][3] + $a[1][1]*$b[1][3] + $a[1][2]*$b[2][3] + $a[1][3]*$b[3][3],
        $a[2][0]*$b[0][0] + $a[2][1]*$b[1][0] + $a[2][2]*$b[2][0] + $a[2][3]*$b[3][0],
        $a[2][0]*$b[0][1] + $a[2][1]*$b[1][1] + $a[2][2]*$b[2][1] + $a[2][3]*$b[3][1],
        $a[2][0]*$b[0][2] + $a[2][1]*$b[1][2] + $a[2][2]*$b[2][2] + $a[2][3]*$b[3][2],
        $a[2][0]*$b[0][3] + $a[2][1]*$b[1][3] + $a[2][2]*$b[2][3] + $a[2][3]*$b[3][3],
        $a[3][0]*$b[0][0] + $a[3][1]*$b[1][0] + $a[3][2]*$b[2][0] + $a[3][3]*$b[3][0],
        $a[3][0]*$b[0][1] + $a[3][1]*$b[1][1] + $a[3][2]*$b[2][1] + $a[3][3]*$b[3][1],
        $a[3][0]*$b[0][2] + $a[3][1]*$b[1][2] + $a[3][2]*$b[2][2] + $a[3][3]*$b[3][2],
        $a[3][0]*$b[0][3] + $a[3][1]*$b[1][3] + $a[3][2]*$b[2][3] + $a[3][3]*$b[3][3]
    );
}

multi sub prod (Mat44:D $a, Mat44:D $b, Mat44:D $r is rw --> Mat44:D) is export {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0]*$b[0][0] + $a[0][1]*$b[1][0] + $a[0][2]*$b[2][0] + $a[0][3]*$b[3][0],
        $a[0][0]*$b[0][1] + $a[0][1]*$b[1][1] + $a[0][2]*$b[2][1] + $a[0][3]*$b[3][1],
        $a[0][0]*$b[0][2] + $a[0][1]*$b[1][2] + $a[0][2]*$b[2][2] + $a[0][3]*$b[3][2],
        $a[0][0]*$b[0][3] + $a[0][1]*$b[1][3] + $a[0][2]*$b[2][3] + $a[0][3]*$b[3][3],
        $a[1][0]*$b[0][0] + $a[1][1]*$b[1][0] + $a[1][2]*$b[2][0] + $a[1][3]*$b[3][0],
        $a[1][0]*$b[0][1] + $a[1][1]*$b[1][1] + $a[1][2]*$b[2][1] + $a[1][3]*$b[3][1],
        $a[1][0]*$b[0][2] + $a[1][1]*$b[1][2] + $a[1][2]*$b[2][2] + $a[1][3]*$b[3][2],
        $a[1][0]*$b[0][3] + $a[1][1]*$b[1][3] + $a[1][2]*$b[2][3] + $a[1][3]*$b[3][3],
        $a[2][0]*$b[0][0] + $a[2][1]*$b[1][0] + $a[2][2]*$b[2][0] + $a[2][3]*$b[3][0],
        $a[2][0]*$b[0][1] + $a[2][1]*$b[1][1] + $a[2][2]*$b[2][1] + $a[2][3]*$b[3][1],
        $a[2][0]*$b[0][2] + $a[2][1]*$b[1][2] + $a[2][2]*$b[2][2] + $a[2][3]*$b[3][2],
        $a[2][0]*$b[0][3] + $a[2][1]*$b[1][3] + $a[2][2]*$b[2][3] + $a[2][3]*$b[3][3],
        $a[3][0]*$b[0][0] + $a[3][1]*$b[1][0] + $a[3][2]*$b[2][0] + $a[3][3]*$b[3][0],
        $a[3][0]*$b[0][1] + $a[3][1]*$b[1][1] + $a[3][2]*$b[2][1] + $a[3][3]*$b[3][1],
        $a[3][0]*$b[0][2] + $a[3][1]*$b[1][2] + $a[3][2]*$b[2][2] + $a[3][3]*$b[3][2],
        $a[3][0]*$b[0][3] + $a[3][1]*$b[1][3] + $a[3][2]*$b[2][3] + $a[3][3]*$b[3][3];
    $r;
}

multi method prod (Mat44:D $a: Mat44:D $b, Mat44:D $r is rw --> Mat44:D) {
    ($r[0][0],$r[0][1],$r[0][2],$r[0][3],$r[1][0],$r[1][1],$r[1][2],$r[1][3],$r[2][0],$r[2][1],$r[2][2],$r[2][3],$r[3][0],$r[3][1],$r[3][2],$r[3][3]) =
        $a[0][0]*$b[0][0] + $a[0][1]*$b[1][0] + $a[0][2]*$b[2][0] + $a[0][3]*$b[3][0],
        $a[0][0]*$b[0][1] + $a[0][1]*$b[1][1] + $a[0][2]*$b[2][1] + $a[0][3]*$b[3][1],
        $a[0][0]*$b[0][2] + $a[0][1]*$b[1][2] + $a[0][2]*$b[2][2] + $a[0][3]*$b[3][2],
        $a[0][0]*$b[0][3] + $a[0][1]*$b[1][3] + $a[0][2]*$b[2][3] + $a[0][3]*$b[3][3],
        $a[1][0]*$b[0][0] + $a[1][1]*$b[1][0] + $a[1][2]*$b[2][0] + $a[1][3]*$b[3][0],
        $a[1][0]*$b[0][1] + $a[1][1]*$b[1][1] + $a[1][2]*$b[2][1] + $a[1][3]*$b[3][1],
        $a[1][0]*$b[0][2] + $a[1][1]*$b[1][2] + $a[1][2]*$b[2][2] + $a[1][3]*$b[3][2],
        $a[1][0]*$b[0][3] + $a[1][1]*$b[1][3] + $a[1][2]*$b[2][3] + $a[1][3]*$b[3][3],
        $a[2][0]*$b[0][0] + $a[2][1]*$b[1][0] + $a[2][2]*$b[2][0] + $a[2][3]*$b[3][0],
        $a[2][0]*$b[0][1] + $a[2][1]*$b[1][1] + $a[2][2]*$b[2][1] + $a[2][3]*$b[3][1],
        $a[2][0]*$b[0][2] + $a[2][1]*$b[1][2] + $a[2][2]*$b[2][2] + $a[2][3]*$b[3][2],
        $a[2][0]*$b[0][3] + $a[2][1]*$b[1][3] + $a[2][2]*$b[2][3] + $a[2][3]*$b[3][3],
        $a[3][0]*$b[0][0] + $a[3][1]*$b[1][0] + $a[3][2]*$b[2][0] + $a[3][3]*$b[3][0],
        $a[3][0]*$b[0][1] + $a[3][1]*$b[1][1] + $a[3][2]*$b[2][1] + $a[3][3]*$b[3][1],
        $a[3][0]*$b[0][2] + $a[3][1]*$b[1][2] + $a[3][2]*$b[2][2] + $a[3][3]*$b[3][2],
        $a[3][0]*$b[0][3] + $a[3][1]*$b[1][3] + $a[3][2]*$b[2][3] + $a[3][3]*$b[3][3];
    $r;
}

multi sub product (Mat44:D $a, Mat44:D $b --> Mat44:D) is export {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0]*$b[0][0] + $a[0][1]*$b[1][0] + $a[0][2]*$b[2][0] + $a[0][3]*$b[3][0],
        $a[0][0]*$b[0][1] + $a[0][1]*$b[1][1] + $a[0][2]*$b[2][1] + $a[0][3]*$b[3][1],
        $a[0][0]*$b[0][2] + $a[0][1]*$b[1][2] + $a[0][2]*$b[2][2] + $a[0][3]*$b[3][2],
        $a[0][0]*$b[0][3] + $a[0][1]*$b[1][3] + $a[0][2]*$b[2][3] + $a[0][3]*$b[3][3],
        $a[1][0]*$b[0][0] + $a[1][1]*$b[1][0] + $a[1][2]*$b[2][0] + $a[1][3]*$b[3][0],
        $a[1][0]*$b[0][1] + $a[1][1]*$b[1][1] + $a[1][2]*$b[2][1] + $a[1][3]*$b[3][1],
        $a[1][0]*$b[0][2] + $a[1][1]*$b[1][2] + $a[1][2]*$b[2][2] + $a[1][3]*$b[3][2],
        $a[1][0]*$b[0][3] + $a[1][1]*$b[1][3] + $a[1][2]*$b[2][3] + $a[1][3]*$b[3][3],
        $a[2][0]*$b[0][0] + $a[2][1]*$b[1][0] + $a[2][2]*$b[2][0] + $a[2][3]*$b[3][0],
        $a[2][0]*$b[0][1] + $a[2][1]*$b[1][1] + $a[2][2]*$b[2][1] + $a[2][3]*$b[3][1],
        $a[2][0]*$b[0][2] + $a[2][1]*$b[1][2] + $a[2][2]*$b[2][2] + $a[2][3]*$b[3][2],
        $a[2][0]*$b[0][3] + $a[2][1]*$b[1][3] + $a[2][2]*$b[2][3] + $a[2][3]*$b[3][3],
        $a[3][0]*$b[0][0] + $a[3][1]*$b[1][0] + $a[3][2]*$b[2][0] + $a[3][3]*$b[3][0],
        $a[3][0]*$b[0][1] + $a[3][1]*$b[1][1] + $a[3][2]*$b[2][1] + $a[3][3]*$b[3][1],
        $a[3][0]*$b[0][2] + $a[3][1]*$b[1][2] + $a[3][2]*$b[2][2] + $a[3][3]*$b[3][2],
        $a[3][0]*$b[0][3] + $a[3][1]*$b[1][3] + $a[3][2]*$b[2][3] + $a[3][3]*$b[3][3];
    $a;
}

multi method product (Mat44:D $a: Mat44:D $b --> Mat44:D) {
    ($a[0][0],$a[0][1],$a[0][2],$a[0][3],$a[1][0],$a[1][1],$a[1][2],$a[1][3],$a[2][0],$a[2][1],$a[2][2],$a[2][3],$a[3][0],$a[3][1],$a[3][2],$a[3][3]) =
        $a[0][0]*$b[0][0] + $a[0][1]*$b[1][0] + $a[0][2]*$b[2][0] + $a[0][3]*$b[3][0],
        $a[0][0]*$b[0][1] + $a[0][1]*$b[1][1] + $a[0][2]*$b[2][1] + $a[0][3]*$b[3][1],
        $a[0][0]*$b[0][2] + $a[0][1]*$b[1][2] + $a[0][2]*$b[2][2] + $a[0][3]*$b[3][2],
        $a[0][0]*$b[0][3] + $a[0][1]*$b[1][3] + $a[0][2]*$b[2][3] + $a[0][3]*$b[3][3],
        $a[1][0]*$b[0][0] + $a[1][1]*$b[1][0] + $a[1][2]*$b[2][0] + $a[1][3]*$b[3][0],
        $a[1][0]*$b[0][1] + $a[1][1]*$b[1][1] + $a[1][2]*$b[2][1] + $a[1][3]*$b[3][1],
        $a[1][0]*$b[0][2] + $a[1][1]*$b[1][2] + $a[1][2]*$b[2][2] + $a[1][3]*$b[3][2],
        $a[1][0]*$b[0][3] + $a[1][1]*$b[1][3] + $a[1][2]*$b[2][3] + $a[1][3]*$b[3][3],
        $a[2][0]*$b[0][0] + $a[2][1]*$b[1][0] + $a[2][2]*$b[2][0] + $a[2][3]*$b[3][0],
        $a[2][0]*$b[0][1] + $a[2][1]*$b[1][1] + $a[2][2]*$b[2][1] + $a[2][3]*$b[3][1],
        $a[2][0]*$b[0][2] + $a[2][1]*$b[1][2] + $a[2][2]*$b[2][2] + $a[2][3]*$b[3][2],
        $a[2][0]*$b[0][3] + $a[2][1]*$b[1][3] + $a[2][2]*$b[2][3] + $a[2][3]*$b[3][3],
        $a[3][0]*$b[0][0] + $a[3][1]*$b[1][0] + $a[3][2]*$b[2][0] + $a[3][3]*$b[3][0],
        $a[3][0]*$b[0][1] + $a[3][1]*$b[1][1] + $a[3][2]*$b[2][1] + $a[3][3]*$b[3][1],
        $a[3][0]*$b[0][2] + $a[3][1]*$b[1][2] + $a[3][2]*$b[2][2] + $a[3][3]*$b[3][2],
        $a[3][0]*$b[0][3] + $a[3][1]*$b[1][3] + $a[3][2]*$b[2][3] + $a[3][3]*$b[3][3];
    $a;
}

