unit class Vec3 is Array;

use Math::ThreeD::Mat44;

method perl () { 'Vec3.new(' ~ join(',', self.list».perl) ~ ')' }

method dump () { say self.perl }

sub vec3 (|a) is export { Vec3.new(|a) }

multi sub vec3-zero () is export { Vec3.new(0,0,0) };

multi sub vec3-ident () is export { Vec3.new(1,1,1) };

multi sub add (Vec3:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] + $b[0],
        $a[1] + $b[1],
        $a[2] + $b[2]
    );
}

multi method add (Vec3:D $a: Vec3:D $b --> Vec3:D) is pure {
    Vec3.new(
        $a[0] + $b[0],
        $a[1] + $b[1],
        $a[2] + $b[2]
    );
}

multi sub add (Vec3:D $a, Vec3:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    ($r[0],$r[1],$r[2]) =
        $a[0] + $b[0],
        $a[1] + $b[1],
        $a[2] + $b[2];
    $r;
}

multi method add (Vec3:D $a: Vec3:D $b, Vec3:D $r is rw --> Vec3:D) {
    ($r[0],$r[1],$r[2]) =
        $a[0] + $b[0],
        $a[1] + $b[1],
        $a[2] + $b[2];
    $r;
}

multi sub plus (Vec3:D $a, Vec3:D $b --> Vec3:D) is export {
    ($a[0],$a[1],$a[2]) =
        $a[0] + $b[0],
        $a[1] + $b[1],
        $a[2] + $b[2];
    $a;
}

multi method plus (Vec3:D $a: Vec3:D $b --> Vec3:D) {
    ($a[0],$a[1],$a[2]) =
        $a[0] + $b[0],
        $a[1] + $b[1],
        $a[2] + $b[2];
    $a;
}

multi sub infix:<+> (Vec3:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] + $b[0],
        $a[1] + $b[1],
        $a[2] + $b[2]
    );
}

multi sub add (Vec3:D $a, Numeric:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] + $b,
        $a[1] + $b,
        $a[2] + $b
    );
}

multi method add (Vec3:D $a: Numeric:D $b --> Vec3:D) is pure {
    Vec3.new(
        $a[0] + $b,
        $a[1] + $b,
        $a[2] + $b
    );
}

multi sub add (Vec3:D $a, Numeric:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    ($r[0],$r[1],$r[2]) =
        $a[0] + $b,
        $a[1] + $b,
        $a[2] + $b;
    $r;
}

multi method add (Vec3:D $a: Numeric:D $b, Vec3:D $r is rw --> Vec3:D) {
    ($r[0],$r[1],$r[2]) =
        $a[0] + $b,
        $a[1] + $b,
        $a[2] + $b;
    $r;
}

multi sub plus (Vec3:D $a, Numeric:D $b --> Vec3:D) is export {
    ($a[0],$a[1],$a[2]) =
        $a[0] + $b,
        $a[1] + $b,
        $a[2] + $b;
    $a;
}

multi method plus (Vec3:D $a: Numeric:D $b --> Vec3:D) {
    ($a[0],$a[1],$a[2]) =
        $a[0] + $b,
        $a[1] + $b,
        $a[2] + $b;
    $a;
}

multi sub infix:<+> (Vec3:D $a, Numeric:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] + $b,
        $a[1] + $b,
        $a[2] + $b
    );
}

multi sub infix:<+> (Numeric:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a + $b[0],
        $a + $b[1],
        $a + $b[2]
    );
}

multi sub sub (Vec3:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] - $b[0],
        $a[1] - $b[1],
        $a[2] - $b[2]
    );
}

multi method sub (Vec3:D $a: Vec3:D $b --> Vec3:D) is pure {
    Vec3.new(
        $a[0] - $b[0],
        $a[1] - $b[1],
        $a[2] - $b[2]
    );
}

multi sub sub (Vec3:D $a, Vec3:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    ($r[0],$r[1],$r[2]) =
        $a[0] - $b[0],
        $a[1] - $b[1],
        $a[2] - $b[2];
    $r;
}

multi method sub (Vec3:D $a: Vec3:D $b, Vec3:D $r is rw --> Vec3:D) {
    ($r[0],$r[1],$r[2]) =
        $a[0] - $b[0],
        $a[1] - $b[1],
        $a[2] - $b[2];
    $r;
}

multi sub minus (Vec3:D $a, Vec3:D $b --> Vec3:D) is export {
    ($a[0],$a[1],$a[2]) =
        $a[0] - $b[0],
        $a[1] - $b[1],
        $a[2] - $b[2];
    $a;
}

multi method minus (Vec3:D $a: Vec3:D $b --> Vec3:D) {
    ($a[0],$a[1],$a[2]) =
        $a[0] - $b[0],
        $a[1] - $b[1],
        $a[2] - $b[2];
    $a;
}

multi sub infix:<-> (Vec3:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] - $b[0],
        $a[1] - $b[1],
        $a[2] - $b[2]
    );
}

multi sub sub (Vec3:D $a, Numeric:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] - $b,
        $a[1] - $b,
        $a[2] - $b
    );
}

multi method sub (Vec3:D $a: Numeric:D $b --> Vec3:D) is pure {
    Vec3.new(
        $a[0] - $b,
        $a[1] - $b,
        $a[2] - $b
    );
}

multi sub sub (Vec3:D $a, Numeric:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    ($r[0],$r[1],$r[2]) =
        $a[0] - $b,
        $a[1] - $b,
        $a[2] - $b;
    $r;
}

multi method sub (Vec3:D $a: Numeric:D $b, Vec3:D $r is rw --> Vec3:D) {
    ($r[0],$r[1],$r[2]) =
        $a[0] - $b,
        $a[1] - $b,
        $a[2] - $b;
    $r;
}

multi sub minus (Vec3:D $a, Numeric:D $b --> Vec3:D) is export {
    ($a[0],$a[1],$a[2]) =
        $a[0] - $b,
        $a[1] - $b,
        $a[2] - $b;
    $a;
}

multi method minus (Vec3:D $a: Numeric:D $b --> Vec3:D) {
    ($a[0],$a[1],$a[2]) =
        $a[0] - $b,
        $a[1] - $b,
        $a[2] - $b;
    $a;
}

multi sub infix:<-> (Vec3:D $a, Numeric:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] - $b,
        $a[1] - $b,
        $a[2] - $b
    );
}

multi sub infix:<-> (Numeric:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a - $b[0],
        $a - $b[1],
        $a - $b[2]
    );
}

multi sub neg (Vec3:D $a --> Vec3:D) is pure is export {
    Vec3.new(
        - $a[0],
        - $a[1],
        - $a[2]
    );
}

multi method neg (Vec3:D $a: --> Vec3:D) is pure {
    Vec3.new(
        - $a[0],
        - $a[1],
        - $a[2]
    );
}

multi sub neg (Vec3:D $a, Vec3:D $r is rw --> Vec3:D) is export {
    ($r[0],$r[1],$r[2]) =
        - $a[0],
        - $a[1],
        - $a[2];
    $r;
}

multi method neg (Vec3:D $a: Vec3:D $r is rw --> Vec3:D) {
    ($r[0],$r[1],$r[2]) =
        - $a[0],
        - $a[1],
        - $a[2];
    $r;
}

multi sub negate (Vec3:D $a --> Vec3:D) is export {
    ($a[0],$a[1],$a[2]) =
        - $a[0],
        - $a[1],
        - $a[2];
    $a;
}

multi method negate (Vec3:D $a: --> Vec3:D) {
    ($a[0],$a[1],$a[2]) =
        - $a[0],
        - $a[1],
        - $a[2];
    $a;
}

multi sub prefix:<-> (Vec3:D $a --> Vec3:D) is pure is export {
    Vec3.new(
        - $a[0],
        - $a[1],
        - $a[2]
    );
}

multi sub mul (Vec3:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] * $b[0],
        $a[1] * $b[1],
        $a[2] * $b[2]
    );
}

multi method mul (Vec3:D $a: Vec3:D $b --> Vec3:D) is pure {
    Vec3.new(
        $a[0] * $b[0],
        $a[1] * $b[1],
        $a[2] * $b[2]
    );
}

multi sub mul (Vec3:D $a, Vec3:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    ($r[0],$r[1],$r[2]) =
        $a[0] * $b[0],
        $a[1] * $b[1],
        $a[2] * $b[2];
    $r;
}

multi method mul (Vec3:D $a: Vec3:D $b, Vec3:D $r is rw --> Vec3:D) {
    ($r[0],$r[1],$r[2]) =
        $a[0] * $b[0],
        $a[1] * $b[1],
        $a[2] * $b[2];
    $r;
}

multi sub times (Vec3:D $a, Vec3:D $b --> Vec3:D) is export {
    ($a[0],$a[1],$a[2]) =
        $a[0] * $b[0],
        $a[1] * $b[1],
        $a[2] * $b[2];
    $a;
}

multi method times (Vec3:D $a: Vec3:D $b --> Vec3:D) {
    ($a[0],$a[1],$a[2]) =
        $a[0] * $b[0],
        $a[1] * $b[1],
        $a[2] * $b[2];
    $a;
}

multi sub infix:<*> (Vec3:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] * $b[0],
        $a[1] * $b[1],
        $a[2] * $b[2]
    );
}

multi sub mul (Vec3:D $a, Numeric:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] * $b,
        $a[1] * $b,
        $a[2] * $b
    );
}

multi method mul (Vec3:D $a: Numeric:D $b --> Vec3:D) is pure {
    Vec3.new(
        $a[0] * $b,
        $a[1] * $b,
        $a[2] * $b
    );
}

multi sub mul (Vec3:D $a, Numeric:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    ($r[0],$r[1],$r[2]) =
        $a[0] * $b,
        $a[1] * $b,
        $a[2] * $b;
    $r;
}

multi method mul (Vec3:D $a: Numeric:D $b, Vec3:D $r is rw --> Vec3:D) {
    ($r[0],$r[1],$r[2]) =
        $a[0] * $b,
        $a[1] * $b,
        $a[2] * $b;
    $r;
}

multi sub times (Vec3:D $a, Numeric:D $b --> Vec3:D) is export {
    ($a[0],$a[1],$a[2]) =
        $a[0] * $b,
        $a[1] * $b,
        $a[2] * $b;
    $a;
}

multi method times (Vec3:D $a: Numeric:D $b --> Vec3:D) {
    ($a[0],$a[1],$a[2]) =
        $a[0] * $b,
        $a[1] * $b,
        $a[2] * $b;
    $a;
}

multi sub infix:<*> (Vec3:D $a, Numeric:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] * $b,
        $a[1] * $b,
        $a[2] * $b
    );
}

multi sub infix:<*> (Numeric:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a * $b[0],
        $a * $b[1],
        $a * $b[2]
    );
}

multi sub div (Vec3:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] / $b[0],
        $a[1] / $b[1],
        $a[2] / $b[2]
    );
}

multi method div (Vec3:D $a: Vec3:D $b --> Vec3:D) is pure {
    Vec3.new(
        $a[0] / $b[0],
        $a[1] / $b[1],
        $a[2] / $b[2]
    );
}

multi sub div (Vec3:D $a, Vec3:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    ($r[0],$r[1],$r[2]) =
        $a[0] / $b[0],
        $a[1] / $b[1],
        $a[2] / $b[2];
    $r;
}

multi method div (Vec3:D $a: Vec3:D $b, Vec3:D $r is rw --> Vec3:D) {
    ($r[0],$r[1],$r[2]) =
        $a[0] / $b[0],
        $a[1] / $b[1],
        $a[2] / $b[2];
    $r;
}

multi sub divide (Vec3:D $a, Vec3:D $b --> Vec3:D) is export {
    ($a[0],$a[1],$a[2]) =
        $a[0] / $b[0],
        $a[1] / $b[1],
        $a[2] / $b[2];
    $a;
}

multi method divide (Vec3:D $a: Vec3:D $b --> Vec3:D) {
    ($a[0],$a[1],$a[2]) =
        $a[0] / $b[0],
        $a[1] / $b[1],
        $a[2] / $b[2];
    $a;
}

multi sub infix:</> (Vec3:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] / $b[0],
        $a[1] / $b[1],
        $a[2] / $b[2]
    );
}

multi sub div (Vec3:D $a, Numeric:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] / $b,
        $a[1] / $b,
        $a[2] / $b
    );
}

multi method div (Vec3:D $a: Numeric:D $b --> Vec3:D) is pure {
    Vec3.new(
        $a[0] / $b,
        $a[1] / $b,
        $a[2] / $b
    );
}

multi sub div (Vec3:D $a, Numeric:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    ($r[0],$r[1],$r[2]) =
        $a[0] / $b,
        $a[1] / $b,
        $a[2] / $b;
    $r;
}

multi method div (Vec3:D $a: Numeric:D $b, Vec3:D $r is rw --> Vec3:D) {
    ($r[0],$r[1],$r[2]) =
        $a[0] / $b,
        $a[1] / $b,
        $a[2] / $b;
    $r;
}

multi sub divide (Vec3:D $a, Numeric:D $b --> Vec3:D) is export {
    ($a[0],$a[1],$a[2]) =
        $a[0] / $b,
        $a[1] / $b,
        $a[2] / $b;
    $a;
}

multi method divide (Vec3:D $a: Numeric:D $b --> Vec3:D) {
    ($a[0],$a[1],$a[2]) =
        $a[0] / $b,
        $a[1] / $b,
        $a[2] / $b;
    $a;
}

multi sub infix:</> (Vec3:D $a, Numeric:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] / $b,
        $a[1] / $b,
        $a[2] / $b
    );
}

multi sub infix:</> (Numeric:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a / $b[0],
        $a / $b[1],
        $a / $b[2]
    );
}

multi sub mod (Vec3:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] % $b[0],
        $a[1] % $b[1],
        $a[2] % $b[2]
    );
}

multi method mod (Vec3:D $a: Vec3:D $b --> Vec3:D) is pure {
    Vec3.new(
        $a[0] % $b[0],
        $a[1] % $b[1],
        $a[2] % $b[2]
    );
}

multi sub mod (Vec3:D $a, Vec3:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    ($r[0],$r[1],$r[2]) =
        $a[0] % $b[0],
        $a[1] % $b[1],
        $a[2] % $b[2];
    $r;
}

multi method mod (Vec3:D $a: Vec3:D $b, Vec3:D $r is rw --> Vec3:D) {
    ($r[0],$r[1],$r[2]) =
        $a[0] % $b[0],
        $a[1] % $b[1],
        $a[2] % $b[2];
    $r;
}

multi sub modulus (Vec3:D $a, Vec3:D $b --> Vec3:D) is export {
    ($a[0],$a[1],$a[2]) =
        $a[0] % $b[0],
        $a[1] % $b[1],
        $a[2] % $b[2];
    $a;
}

multi method modulus (Vec3:D $a: Vec3:D $b --> Vec3:D) {
    ($a[0],$a[1],$a[2]) =
        $a[0] % $b[0],
        $a[1] % $b[1],
        $a[2] % $b[2];
    $a;
}

multi sub infix:<%> (Vec3:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] % $b[0],
        $a[1] % $b[1],
        $a[2] % $b[2]
    );
}

multi sub mod (Vec3:D $a, Numeric:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] % $b,
        $a[1] % $b,
        $a[2] % $b
    );
}

multi method mod (Vec3:D $a: Numeric:D $b --> Vec3:D) is pure {
    Vec3.new(
        $a[0] % $b,
        $a[1] % $b,
        $a[2] % $b
    );
}

multi sub mod (Vec3:D $a, Numeric:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    ($r[0],$r[1],$r[2]) =
        $a[0] % $b,
        $a[1] % $b,
        $a[2] % $b;
    $r;
}

multi method mod (Vec3:D $a: Numeric:D $b, Vec3:D $r is rw --> Vec3:D) {
    ($r[0],$r[1],$r[2]) =
        $a[0] % $b,
        $a[1] % $b,
        $a[2] % $b;
    $r;
}

multi sub modulus (Vec3:D $a, Numeric:D $b --> Vec3:D) is export {
    ($a[0],$a[1],$a[2]) =
        $a[0] % $b,
        $a[1] % $b,
        $a[2] % $b;
    $a;
}

multi method modulus (Vec3:D $a: Numeric:D $b --> Vec3:D) {
    ($a[0],$a[1],$a[2]) =
        $a[0] % $b,
        $a[1] % $b,
        $a[2] % $b;
    $a;
}

multi sub infix:<%> (Vec3:D $a, Numeric:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0] % $b,
        $a[1] % $b,
        $a[2] % $b
    );
}

multi sub infix:<%> (Numeric:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a % $b[0],
        $a % $b[1],
        $a % $b[2]
    );
}

multi sub postfix:<!> (Vec3:D $a --> Vec3:D) is pure is export {
    Vec3.new(
        ([*] 1..$a[0]),
        ([*] 1..$a[1]),
        ([*] 1..$a[2])
    );
}

multi sub cross-with (Vec3:D $a, Vec3:D $b --> Vec3:D) is export {
    ($a[0],$a[1],$a[2]) =
        $a[1] * $b[2] - $a[2] * $b[1],
        $a[2] * $b[0] - $a[0] * $b[2],
        $a[0] * $b[1] - $a[1] * $b[0];
    $a;
}

multi method cross-with (Vec3:D $a: Vec3:D $b --> Vec3:D) {
    ($a[0],$a[1],$a[2]) =
        $a[1] * $b[2] - $a[2] * $b[1],
        $a[2] * $b[0] - $a[0] * $b[2],
        $a[0] * $b[1] - $a[1] * $b[0];
    $a;
}

multi sub infix:<⨯> (Vec3:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[1] * $b[2] - $a[2] * $b[1],
        $a[2] * $b[0] - $a[0] * $b[2],
        $a[0] * $b[1] - $a[1] * $b[0]
    );
}

multi sub dot (Vec3:D $a, Vec3:D $b --> Numeric:D) is pure is export {
    $a[0]*$b[0] + $a[1]*$b[1] + $a[2]*$b[2]
}

multi method dot (Vec3:D $a: Vec3:D $b --> Numeric:D) is pure {
    $a[0]*$b[0] + $a[1]*$b[1] + $a[2]*$b[2]
}

multi sub dot (Vec3:D $a, Vec3:D $b, Numeric:D $r is rw --> Numeric:D) is export {
    $r = $a[0]*$b[0] + $a[1]*$b[1] + $a[2]*$b[2];
}

multi method dot (Vec3:D $a: Vec3:D $b, Numeric:D $r is rw --> Numeric:D) {
    $r = $a[0]*$b[0] + $a[1]*$b[1] + $a[2]*$b[2];
}

multi sub infix:<⋅> (Vec3:D $a, Vec3:D $b --> Numeric:D) is pure is export {
    $a[0]*$b[0] + $a[1]*$b[1] + $a[2]*$b[2]
}

multi sub length_sqr (Vec3:D $a --> Numeric:D) is pure is export {
    $a[0]*$a[0] + $a[1]*$a[1] + $a[2]*$a[2]
}

multi method length_sqr (Vec3:D $a: --> Numeric:D) is pure {
    $a[0]*$a[0] + $a[1]*$a[1] + $a[2]*$a[2]
}

multi sub length_sqr (Vec3:D $a, Numeric:D $r is rw --> Numeric:D) is export {
    $r = $a[0]*$a[0] + $a[1]*$a[1] + $a[2]*$a[2];
}

multi method length_sqr (Vec3:D $a: Numeric:D $r is rw --> Numeric:D) {
    $r = $a[0]*$a[0] + $a[1]*$a[1] + $a[2]*$a[2];
}

multi sub length (Vec3:D $a --> Numeric:D) is pure is export {
    sqrt( $a[0]*$a[0] + $a[1]*$a[1] + $a[2]*$a[2] )
}

multi method length (Vec3:D $a: --> Numeric:D) is pure {
    sqrt( $a[0]*$a[0] + $a[1]*$a[1] + $a[2]*$a[2] )
}

multi sub length (Vec3:D $a, Numeric:D $r is rw --> Numeric:D) is export {
    $r = sqrt( $a[0]*$a[0] + $a[1]*$a[1] + $a[2]*$a[2] );
}

multi method length (Vec3:D $a: Numeric:D $r is rw --> Numeric:D) {
    $r = sqrt( $a[0]*$a[0] + $a[1]*$a[1] + $a[2]*$a[2] );
}

multi sub rot-x (Vec3:D $a, Numeric:D $b --> Vec3:D) is pure is export {
    my ($sin, $cos) = sin($b), cos($b);

    Vec3.new(
        $a[0],
        $a[1] * $cos - $a[2] * $sin,
        $a[1] * $sin + $a[2] * $cos
    );
}

multi method rot-x (Vec3:D $a: Numeric:D $b --> Vec3:D) is pure {
    my ($sin, $cos) = sin($b), cos($b);

    Vec3.new(
        $a[0],
        $a[1] * $cos - $a[2] * $sin,
        $a[1] * $sin + $a[2] * $cos
    );
}

multi sub rot-x (Vec3:D $a, Numeric:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    my ($sin, $cos) = sin($b), cos($b);

    ($r[0],$r[1],$r[2]) =
        $a[0],
        $a[1] * $cos - $a[2] * $sin,
        $a[1] * $sin + $a[2] * $cos;
    $r;
}

multi method rot-x (Vec3:D $a: Numeric:D $b, Vec3:D $r is rw --> Vec3:D) {
    my ($sin, $cos) = sin($b), cos($b);

    ($r[0],$r[1],$r[2]) =
        $a[0],
        $a[1] * $cos - $a[2] * $sin,
        $a[1] * $sin + $a[2] * $cos;
    $r;
}

multi sub rotate-x (Vec3:D $a, Numeric:D $b --> Vec3:D) is export {
    my ($sin, $cos) = sin($b), cos($b);

    ($a[0],$a[1],$a[2]) =
        $a[0],
        $a[1] * $cos - $a[2] * $sin,
        $a[1] * $sin + $a[2] * $cos;
    $a;
}

multi method rotate-x (Vec3:D $a: Numeric:D $b --> Vec3:D) {
    my ($sin, $cos) = sin($b), cos($b);

    ($a[0],$a[1],$a[2]) =
        $a[0],
        $a[1] * $cos - $a[2] * $sin,
        $a[1] * $sin + $a[2] * $cos;
    $a;
}

multi sub rot-y (Vec3:D $a, Numeric:D $b --> Vec3:D) is pure is export {
    my ($sin, $cos) = sin($b), cos($b);

    Vec3.new(
        $a[0] * $cos - $a[2] * $sin,
        $a[1],
        $a[0] * $sin + $a[2] * $cos
    );
}

multi method rot-y (Vec3:D $a: Numeric:D $b --> Vec3:D) is pure {
    my ($sin, $cos) = sin($b), cos($b);

    Vec3.new(
        $a[0] * $cos - $a[2] * $sin,
        $a[1],
        $a[0] * $sin + $a[2] * $cos
    );
}

multi sub rot-y (Vec3:D $a, Numeric:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    my ($sin, $cos) = sin($b), cos($b);

    ($r[0],$r[1],$r[2]) =
        $a[0] * $cos - $a[2] * $sin,
        $a[1],
        $a[0] * $sin + $a[2] * $cos;
    $r;
}

multi method rot-y (Vec3:D $a: Numeric:D $b, Vec3:D $r is rw --> Vec3:D) {
    my ($sin, $cos) = sin($b), cos($b);

    ($r[0],$r[1],$r[2]) =
        $a[0] * $cos - $a[2] * $sin,
        $a[1],
        $a[0] * $sin + $a[2] * $cos;
    $r;
}

multi sub rotate-y (Vec3:D $a, Numeric:D $b --> Vec3:D) is export {
    my ($sin, $cos) = sin($b), cos($b);

    ($a[0],$a[1],$a[2]) =
        $a[0] * $cos - $a[2] * $sin,
        $a[1],
        $a[0] * $sin + $a[2] * $cos;
    $a;
}

multi method rotate-y (Vec3:D $a: Numeric:D $b --> Vec3:D) {
    my ($sin, $cos) = sin($b), cos($b);

    ($a[0],$a[1],$a[2]) =
        $a[0] * $cos - $a[2] * $sin,
        $a[1],
        $a[0] * $sin + $a[2] * $cos;
    $a;
}

multi sub rot-z (Vec3:D $a, Numeric:D $b --> Vec3:D) is pure is export {
    my ($sin, $cos) = sin($b), cos($b);

    Vec3.new(
        $a[0] * $cos - $a[1] * $sin,
        $a[0] * $sin + $a[1] * $cos,
        $a[2]
    );
}

multi method rot-z (Vec3:D $a: Numeric:D $b --> Vec3:D) is pure {
    my ($sin, $cos) = sin($b), cos($b);

    Vec3.new(
        $a[0] * $cos - $a[1] * $sin,
        $a[0] * $sin + $a[1] * $cos,
        $a[2]
    );
}

multi sub rot-z (Vec3:D $a, Numeric:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    my ($sin, $cos) = sin($b), cos($b);

    ($r[0],$r[1],$r[2]) =
        $a[0] * $cos - $a[1] * $sin,
        $a[0] * $sin + $a[1] * $cos,
        $a[2];
    $r;
}

multi method rot-z (Vec3:D $a: Numeric:D $b, Vec3:D $r is rw --> Vec3:D) {
    my ($sin, $cos) = sin($b), cos($b);

    ($r[0],$r[1],$r[2]) =
        $a[0] * $cos - $a[1] * $sin,
        $a[0] * $sin + $a[1] * $cos,
        $a[2];
    $r;
}

multi sub rotate-z (Vec3:D $a, Numeric:D $b --> Vec3:D) is export {
    my ($sin, $cos) = sin($b), cos($b);

    ($a[0],$a[1],$a[2]) =
        $a[0] * $cos - $a[1] * $sin,
        $a[0] * $sin + $a[1] * $cos,
        $a[2];
    $a;
}

multi method rotate-z (Vec3:D $a: Numeric:D $b --> Vec3:D) {
    my ($sin, $cos) = sin($b), cos($b);

    ($a[0],$a[1],$a[2]) =
        $a[0] * $cos - $a[1] * $sin,
        $a[0] * $sin + $a[1] * $cos,
        $a[2];
    $a;
}

multi sub rot (Vec3:D $a, Vec3:D $b, Numeric:D $c --> Vec3:D) is pure is export {
    my $sin = sin $c;
    my $cos = cos $c;
    my $dot_scaled = $b.dot($a) * (1 - $cos);
    my $cross = $b.cross($a);

    Vec3.new(
        $a[0] * $cos + $cross[0] * $sin + $b[0] * $dot_scaled,
        $a[1] * $cos + $cross[1] * $sin + $b[1] * $dot_scaled,
        $a[2] * $cos + $cross[2] * $sin + $b[2] * $dot_scaled
    );
}

multi method rot (Vec3:D $a: Vec3:D $b, Numeric:D $c --> Vec3:D) is pure {
    my $sin = sin $c;
    my $cos = cos $c;
    my $dot_scaled = $b.dot($a) * (1 - $cos);
    my $cross = $b.cross($a);

    Vec3.new(
        $a[0] * $cos + $cross[0] * $sin + $b[0] * $dot_scaled,
        $a[1] * $cos + $cross[1] * $sin + $b[1] * $dot_scaled,
        $a[2] * $cos + $cross[2] * $sin + $b[2] * $dot_scaled
    );
}

multi sub rot (Vec3:D $a, Vec3:D $b, Numeric:D $c, Vec3:D $r is rw --> Vec3:D) is export {
    my $sin = sin $c;
    my $cos = cos $c;
    my $dot_scaled = $b.dot($a) * (1 - $cos);
    my $cross = $b.cross($a);

    ($r[0],$r[1],$r[2]) =
        $a[0] * $cos + $cross[0] * $sin + $b[0] * $dot_scaled,
        $a[1] * $cos + $cross[1] * $sin + $b[1] * $dot_scaled,
        $a[2] * $cos + $cross[2] * $sin + $b[2] * $dot_scaled;
    $r;
}

multi method rot (Vec3:D $a: Vec3:D $b, Numeric:D $c, Vec3:D $r is rw --> Vec3:D) {
    my $sin = sin $c;
    my $cos = cos $c;
    my $dot_scaled = $b.dot($a) * (1 - $cos);
    my $cross = $b.cross($a);

    ($r[0],$r[1],$r[2]) =
        $a[0] * $cos + $cross[0] * $sin + $b[0] * $dot_scaled,
        $a[1] * $cos + $cross[1] * $sin + $b[1] * $dot_scaled,
        $a[2] * $cos + $cross[2] * $sin + $b[2] * $dot_scaled;
    $r;
}

multi sub rotate (Vec3:D $a, Vec3:D $b, Numeric:D $c --> Vec3:D) is export {
    my $sin = sin $c;
    my $cos = cos $c;
    my $dot_scaled = $b.dot($a) * (1 - $cos);
    my $cross = $b.cross($a);

    ($a[0],$a[1],$a[2]) =
        $a[0] * $cos + $cross[0] * $sin + $b[0] * $dot_scaled,
        $a[1] * $cos + $cross[1] * $sin + $b[1] * $dot_scaled,
        $a[2] * $cos + $cross[2] * $sin + $b[2] * $dot_scaled;
    $a;
}

multi method rotate (Vec3:D $a: Vec3:D $b, Numeric:D $c --> Vec3:D) {
    my $sin = sin $c;
    my $cos = cos $c;
    my $dot_scaled = $b.dot($a) * (1 - $cos);
    my $cross = $b.cross($a);

    ($a[0],$a[1],$a[2]) =
        $a[0] * $cos + $cross[0] * $sin + $b[0] * $dot_scaled,
        $a[1] * $cos + $cross[1] * $sin + $b[1] * $dot_scaled,
        $a[2] * $cos + $cross[2] * $sin + $b[2] * $dot_scaled;
    $a;
}

multi sub trans (Vec3:D $a, Mat44:D $b --> Vec3:D) is pure is export {
    Vec3.new(
        $a[0]*$b[0][0] + $a[1]*$b[0][1] + $a[2]*$b[0][2] + $b[0][3],
        $a[0]*$b[1][0] + $a[1]*$b[1][1] + $a[2]*$b[1][2] + $b[1][3],
        $a[0]*$b[2][0] + $a[1]*$b[2][1] + $a[2]*$b[2][2] + $b[2][3]
    );
}

multi method trans (Vec3:D $a: Mat44:D $b --> Vec3:D) is pure {
    Vec3.new(
        $a[0]*$b[0][0] + $a[1]*$b[0][1] + $a[2]*$b[0][2] + $b[0][3],
        $a[0]*$b[1][0] + $a[1]*$b[1][1] + $a[2]*$b[1][2] + $b[1][3],
        $a[0]*$b[2][0] + $a[1]*$b[2][1] + $a[2]*$b[2][2] + $b[2][3]
    );
}

multi sub trans (Vec3:D $a, Mat44:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    ($r[0],$r[1],$r[2]) =
        $a[0]*$b[0][0] + $a[1]*$b[0][1] + $a[2]*$b[0][2] + $b[0][3],
        $a[0]*$b[1][0] + $a[1]*$b[1][1] + $a[2]*$b[1][2] + $b[1][3],
        $a[0]*$b[2][0] + $a[1]*$b[2][1] + $a[2]*$b[2][2] + $b[2][3];
    $r;
}

multi method trans (Vec3:D $a: Mat44:D $b, Vec3:D $r is rw --> Vec3:D) {
    ($r[0],$r[1],$r[2]) =
        $a[0]*$b[0][0] + $a[1]*$b[0][1] + $a[2]*$b[0][2] + $b[0][3],
        $a[0]*$b[1][0] + $a[1]*$b[1][1] + $a[2]*$b[1][2] + $b[1][3],
        $a[0]*$b[2][0] + $a[1]*$b[2][1] + $a[2]*$b[2][2] + $b[2][3];
    $r;
}

multi sub transform (Vec3:D $a, Mat44:D $b --> Vec3:D) is export {
    ($a[0],$a[1],$a[2]) =
        $a[0]*$b[0][0] + $a[1]*$b[0][1] + $a[2]*$b[0][2] + $b[0][3],
        $a[0]*$b[1][0] + $a[1]*$b[1][1] + $a[2]*$b[1][2] + $b[1][3],
        $a[0]*$b[2][0] + $a[1]*$b[2][1] + $a[2]*$b[2][2] + $b[2][3];
    $a;
}

multi method transform (Vec3:D $a: Mat44:D $b --> Vec3:D) {
    ($a[0],$a[1],$a[2]) =
        $a[0]*$b[0][0] + $a[1]*$b[0][1] + $a[2]*$b[0][2] + $b[0][3],
        $a[0]*$b[1][0] + $a[1]*$b[1][1] + $a[2]*$b[1][2] + $b[1][3],
        $a[0]*$b[2][0] + $a[1]*$b[2][1] + $a[2]*$b[2][2] + $b[2][3];
    $a;
}

multi sub norm (Vec3:D $a --> Vec3:D) is pure is export {
    my $l = $a.length || 1;

    Vec3.new(
        $a[0] / $l,
        $a[1] / $l,
        $a[2] / $l
    );
}

multi method norm (Vec3:D $a: --> Vec3:D) is pure {
    my $l = $a.length || 1;

    Vec3.new(
        $a[0] / $l,
        $a[1] / $l,
        $a[2] / $l
    );
}

multi sub norm (Vec3:D $a, Vec3:D $r is rw --> Vec3:D) is export {
    my $l = $a.length || 1;

    ($r[0],$r[1],$r[2]) =
        $a[0] / $l,
        $a[1] / $l,
        $a[2] / $l;
    $r;
}

multi method norm (Vec3:D $a: Vec3:D $r is rw --> Vec3:D) {
    my $l = $a.length || 1;

    ($r[0],$r[1],$r[2]) =
        $a[0] / $l,
        $a[1] / $l,
        $a[2] / $l;
    $r;
}

multi sub normalize (Vec3:D $a --> Vec3:D) is export {
    my $l = $a.length || 1;

    ($a[0],$a[1],$a[2]) =
        $a[0] / $l,
        $a[1] / $l,
        $a[2] / $l;
    $a;
}

multi method normalize (Vec3:D $a: --> Vec3:D) {
    my $l = $a.length || 1;

    ($a[0],$a[1],$a[2]) =
        $a[0] / $l,
        $a[1] / $l,
        $a[2] / $l;
    $a;
}

multi sub refl (Vec3:D $a, Vec3:D $b --> Vec3:D) is pure is export {
    my $scale = 2 * $a.dot($b);

    Vec3.new(
        $b[0] * $scale - $a[0],
        $b[1] * $scale - $a[1],
        $b[2] * $scale - $a[2]
    );
}

multi method refl (Vec3:D $a: Vec3:D $b --> Vec3:D) is pure {
    my $scale = 2 * $a.dot($b);

    Vec3.new(
        $b[0] * $scale - $a[0],
        $b[1] * $scale - $a[1],
        $b[2] * $scale - $a[2]
    );
}

multi sub refl (Vec3:D $a, Vec3:D $b, Vec3:D $r is rw --> Vec3:D) is export {
    my $scale = 2 * $a.dot($b);

    ($r[0],$r[1],$r[2]) =
        $b[0] * $scale - $a[0],
        $b[1] * $scale - $a[1],
        $b[2] * $scale - $a[2];
    $r;
}

multi method refl (Vec3:D $a: Vec3:D $b, Vec3:D $r is rw --> Vec3:D) {
    my $scale = 2 * $a.dot($b);

    ($r[0],$r[1],$r[2]) =
        $b[0] * $scale - $a[0],
        $b[1] * $scale - $a[1],
        $b[2] * $scale - $a[2];
    $r;
}

multi sub reflect (Vec3:D $a, Vec3:D $b --> Vec3:D) is export {
    my $scale = 2 * $a.dot($b);

    ($a[0],$a[1],$a[2]) =
        $b[0] * $scale - $a[0],
        $b[1] * $scale - $a[1],
        $b[2] * $scale - $a[2];
    $a;
}

multi method reflect (Vec3:D $a: Vec3:D $b --> Vec3:D) {
    my $scale = 2 * $a.dot($b);

    ($a[0],$a[1],$a[2]) =
        $b[0] * $scale - $a[0],
        $b[1] * $scale - $a[1],
        $b[2] * $scale - $a[2];
    $a;
}

